#
# Enhydra Java Application Server 
# The Initial Developer of the Original Code is Lutris Technologies Inc. 
# Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
# Inc. 
# All Rights Reserved. 
#
# The contents of this file are subject to the Enhydra Public License Version
# 1.0 (the "License"); you may not use this file except in compliance with the
# License. You may obtain a copy of the License at
# http://www.enhydra.org/software/license/epl.html 
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations under the
# License. 
#
# $Id: acinclude.m4,v 1.1 2003/04/14 12:25:53 sinisa Exp $
#

#
# Borrowed from the Apache/JServ:
#
# configure.in file for Apache Jserv -*-sh-*-
# Copyright (c) 1997-1998 The Java Apache Project. All rights reserved.    
# Author:  Tommy Reilly <tpr@neog.com>
#
# $Id: acinclude.m4,v 1.1 2003/04/14 12:25:53 sinisa Exp $
#

AC_DEFUN(EDIR_CHECK_APACHE, [
    AC_MSG_CHECKING(for apxs)
    AC_PATH_PROG(XX_APXS,apxs)
    AC_ARG_WITH(apxs,
    [  --with-apxs=APXS-UTIL   tells configure where to look
			  for the Apache apxs utility (for 1.3 DSO build)],
    [
	# we test for it actually being valid below...
	APXS_CMD="${withval}"
    ],
    [
	dnl Too bad, will have to guess the location

	AC_PATH_PROG(TEST,test,$PATH)dnl
	
	if ${TEST} ! -n ""${XX_APXS} ; then
	    for APXS_GUESS in \
		/usr/local/apache/sbin \
		/usr/local/apache/bin \
		/usr/local/sbin \
		/usr/sbin \
		/usr/bin \
		/u/http/common/bin
	    do
		if ${TEST} -x ${APXS_GUESS}/apxs ; then
		    APXS_CMD=${APXS_GUESS}/apxs
		    break
		fi
	    done
	else
	    APXS_CMD=${XX_APXS}
	fi
    ])    

    if ${TEST} -n "${APXS_CMD}" ; then

	AC_MSG_RESULT([need to check for Perl first, apxs depends on it...])
	AC_PATH_PROG(PERL,perl,$PATH)dnl

	dnl Seems that we have it, but have to check if it is OK first
	
	if ${TEST} ! -x "${APXS_CMD}" ; then
	    AC_MSG_ERROR(Invalid location for apxs: '${APXS_CMD}')
	fi
	
	$APXS_CMD -q PREFIX >/dev/null 2>/dev/null || apxs_support=false

	if ${TEST} "${apxs_support}" = "false" ; then
	    AC_MSG_RESULT(could not find apxs)
	    AC_MSG_ERROR(You must specify a valid --with-apxs path)
	else
	    APXS=$APXS_CMD
	    AC_SUBST(APXS)
	    AC_MSG_RESULT([${APXS}, assume Apache 1.3 with DSO support])
	fi
    else
	AC_MSG_ERROR([Could not find the Apache "apxs" program.\nYou must specify the --with-apxs configure option to let configure \nknow where to find the apache installation program])
    fi

    # VT: At this point, I'm perfectly sure that I have ${APXS} properly
    # set, or not, so I can forget about trying to figure out where it is

    AC_MSG_CHECKING([for Apache installation directory (assume shared build)])
    apache_dir=`$APXS -q PREFIX`
    AC_MSG_RESULT(${apache_dir})

    AC_MSG_CHECKING(for Apache include directory)
    apache_include="`$APXS -q INCLUDEDIR`"
    AC_MSG_RESULT(${apache_include})

    AC_MSG_CHECKING(for Apache configuration directory)
    sysconfdir=`$APXS -q SYSCONFDIR`
    AC_MSG_RESULT(${sysconfdir})

    AC_MSG_CHECKING(for Apache library directory)
    libexecdir=`$APXS -q LIBEXECDIR`
    libdir=/tmp/edir_lib
    AC_MSG_RESULT(${libexecdir})

    AC_MSG_CHECKING(for Apache program directory)
    sbindir=`$APXS -q SBINDIR`
    AC_MSG_RESULT(${apache_sbindir})

    # Add a test to check for a broken RH 6.1 RPM installation.
    # apxs incorrectly returns "modules" instead of the right path.
    if ${TEST} "${libexecdir}" = "modules" ; then
	AC_MSG_ERROR([You have a broken RedHat Apache RPM. Please 
	download the latest Apache source distribution from 
	www.apache.org and build using that.])
    fi

    AC_MSG_CHECKING(for Apache layout consistency)

    for testdir in ${apache_dir} ${apache_include} ${libexecdir} ${sysconfigdir} ; do
	if ${TEST} ! -d ${testdir} ; then
	    AC_MSG_ERROR(${testdir} doesn't exist, not readable or not a directory)
	fi
    done

    # Now, I'm safe, all the directories exist (should I have
    # checked them for readability?)

    # preprocessor option for include
    apache_include="-I${apache_include}"

    APACHE_INCLUDE=${apache_include}
    APACHE_DIR=${apache_dir}
    LIBEXECDIR=`eval echo "${libexecdir}"`
    LIBDIR=`eval echo "${libdir}"`
    APACHE_SBIN=`eval echo "${sbindir}"`
    SYSCONFDIR=`eval echo "${sysconfdir}"`
    DEF_DATA='"'`eval echo "${apache_dir}/logs/edir/enhydra_director.data"`'"'
    DEF_LOCK='"'`eval echo "${apache_dir}/logs/edir/enhydra_director.ipc"`'"'
    DEF_DAEMON='"'`eval echo "${sbindir}"`'"'
    DEF_CONF='"'`eval echo "${sysconfdir}"`'"'

    AC_DEFINE_UNQUOTED(EDIR_AC_DEF_SB_DATA_FILE, ${DEF_DATA})
    AC_DEFINE_UNQUOTED(EDIR_AC_DEF_SB_LOCK_FILE, ${DEF_LOCK})
    AC_DEFINE_UNQUOTED(EDIR_AC_DAEMON_DIR, ${DEF_DAEMON})
    AC_DEFINE_UNQUOTED(EDIR_AC_CONFIG_DIR, ${DEF_CONF})

    AC_SUBST(APACHE_INCLUDE)
    AC_SUBST(APACHE_DIR)
    AC_SUBST(LIBEXECDIR)
    AC_SUBST(APACHE_SBIN)
    AC_SUBST(SYSCONFDIR)
])
