#ifndef __MOD_ENHYDRA_DIRECTOR_H__
#define __MOD_ENHYDRA_DIRECTOR_H__ 1
#ifdef __cplusplus
extern "C" {
#endif

#include "httpd.h"
#include "http_config.h"

/**
 * Enhydra Director configuration info.
 */
typedef struct edir_apache_cfg {
	/*** Bookeeping to keep track of where in Apache config we are    ***/
	int   cfg_Mode;	   /* Per-server or Per-directory config?		    */
	int   inited;	   /* Whether we've initialized yet.                */
	int   cfg_Inheritable; /* Can virt. servers inherit this from main? */
	char  *cfg_Server;	   /* Server name if per-virtual-host config    */
	char  *cfg_Dir;	   /* Dir path if per-directory config              */
    /*** Per-instance config for each Enhydra Director setup.         ***/
	int32 inBufSize;
	int32 outBufSize;
	char  *configFile;     /* Configuration file for this instance      */
// Apache/NT is multithreaded, so we can use global scoreboard variable
//    char  *dataFile;       /* Optional place to store scoreboard data   */
//                           /*   NOTE: This _may_ be an IPC key #        */
//    char  *lockFile;       /* Optional place to locate locks/mutexes    */
//                           /*   NOTE: This _may_ be an IPC key #        */
// No need for daemon, also
//    char  *pathToDaemon;   /* Optional full path to daemon executable.  */
	char  *sessionIdPrefix;/* Prefix of Session ID cookie or component  */
// Apache/NT is multithreaded, so we don't need process handles
//	edir_load_handles_t
//			*handles;        /* Scoreboard handles. */
} edir_apache_cfg_t;

/**
 * Structure to pass to edir_parse_uri() to receive important URI
 * fragments.
 */
typedef struct edir_uri_data {
	char *uriBuf;	   	/* destroyed uri buffer */
	char *pathInfo;	   	/* Stripped, normalized URI path. */
	char *pathParams;	   	/* Full path parameters string, if any */
	char *sessionID;       	/* Session ID, if any.  */
	char *queryString;     	/* Query String, if any. */
	edir_kvtable_t *cookies;	/* Hack: let cookie getter set sessionID */
	pool *pool;		   	/* Hack: cookie getter needs this */
} edir_uri_data_t;

#define DIR_CONFIG(R) \
	((edir_apache_cfg_t *) \
	  (ap_get_module_config((R)->per_dir_config, &enhydra_director_module)));

#define SERVER_CONFIG(S) \
	((edir_apache_cfg_t *) \
	  (ap_get_module_config((S)->module_config, &enhydra_director_module)));

#define REQ_CONFIG(R) \
	((edir_apache_cfg_t *) \
	  (ap_get_module_config((R)->request_config, &enhydra_director_module)));


#define EDIR_APACHE_PREBUF_SIZE		(2048)
#define EDIR_APACHE_INBUF_SIZE		(4096)
#define EDIR_APACHE_OUTBUF_SIZE		(4096)

#define EDIR_APACHE_CFG_MODE_SERVER	(1)
#define EDIR_APACHE_CFG_MODE_DIR		(2)

extern int32					*refCounts;
extern int32					numServers;
extern edir_load_sb_offsets_t	*scoreboard;

extern pool *edir_apache_pool;
//extern pool *edir_apache_pool_request;
extern pool *edir_apache_pool_current;

extern char * DefaultConfigFile;

extern CRITICAL_SECTION vLock;


extern void edir_apache_master_init(server_rec *s, pool *p);
extern void edir_apache_init(server_rec *s, pool *p);
extern void edir_apache_child_exit(server_rec *s, pool *p);
extern void *edir_apache_create_dir_config(pool *p, char *dirspec);
extern void *edir_apache_create_server_config(pool *p, server_rec *s);

extern module MODULE_VAR_EXPORT enhydra_director_module;

#ifdef __cplusplus
} /* Extern "C" */
#endif
#endif /* __MOD_ENHYDRA_DIRECTOR_H__ */
