#include "edir.h"
#include "httpd.h"
#include "http_config.h"

#include "mod_enhydra_director.h"
#include "mod_enhydra_director_cmds.h"

/**
 * Command parse handler for the 'EnhydraDirectorVirtualInherit' command. (FLAG)
 *
 * @param cmd     [in]     Apache command parsing context handle.
 * @param notused [unused] Unused Parameter.
 * @param bool    [in]     Boolean configuration value to set.
 * @return An error message, if a misconfiguration is detected.
 */
static const char *
cmd_Inherit(cmd_parms *cmd, void *notused, int bool)
{
	edir_apache_cfg_t *cfg = SERVER_CONFIG(cmd->server);
	cfg->cfg_Inheritable = bool;
	return NULL;
}

/**
 * Command parse handler for the 'EnhydraDirectorConfigFile' command. (TAKE1)
 *
 * @param cmd     [in]     Apache command parsing context handle.
 * @param notused [unused] Unused Parameter.
 * @param value   [in]     String configuration value to set.
 * @return An error message, if a misconfiguration is detected.
 */
static const char *
cmd_ConfigFile(cmd_parms *cmd, void *notused, char *value)
{
	edir_apache_cfg_t *cfg = SERVER_CONFIG(cmd->server);
	cfg->configFile = value ? ap_pstrdup(cmd->pool, value) : NULL;
	return NULL;
}

/**
 * Command parse handler for the 'EnhydraDirectorDataFile' command. (TAKE1)
 * 
 * @param cmd     [in]     Apache command parsing context handle.
 * @param notused [unused] Unused Parameter.
 * @param value   [in]     String configuration value to set.
 * @return An error message, if a misconfiguration is detected.
 *
 * Does nothing on NT !
 */
static const char *
cmd_DataFile(cmd_parms *cmd, void *notused, char *value)
{
	return NULL;
}

/**
 * Command parse handler for the 'EnhydraDirectorLockFile' command. (TAKE1)
 *
 * @param cmd     [in]     Apache command parsing context handle.
 * @param notused [unused] Unused Parameter.
 * @param value   [in]     String configuration value to set.
 * @return An error message, if a misconfiguration is detected.
 *
 * Does nothing on NT !
 */
static const char *
cmd_LockFile(cmd_parms *cmd, void *notused, char *value)
{
	return NULL;
}

/**
 * Command parse handler for the 'EnhydraDirectorDaemonPath' command. (TAKE1)
 *
 * @param cmd     [in]     Apache command parsing context handle.
 * @param notused [unused] Unused Parameter.
 * @param value   [in]     String configuration value to set.
 * @return An error message, if a misconfiguration is detected.
 *
 * Does nothing on NT !
 */
static const char *
cmd_DaemonPath(cmd_parms *cmd, void *notused, char *value)
{
	return NULL;
}

/**
 * Command parse handler for the 'EnhydraDirectorDebug' command. (TAKE1)
 *
 * @param cmd     [in]     Apache command parsing context handle.
 * @param notused [unused] Unused Parameter.
 * @param value   [in]     String configuration value to set.
 * @return An error message, if a misconfiguration is detected.
 */
static const char *
cmd_Debug(cmd_parms *cmd, void *notused, char *value)
{
	edir_atoi(value, &edir_debug);
	return NULL;
}

/*
 * List of configuration file directives known to Enhydra Director.
 * <P>
 * These directives are only alowed at the top level resource level,
 * and are not allowed within directory configuration.  Enhydra Director
 * dynamically figures out prefix to handler mappings from its own
 * configuration file.
 *
 * @see edir_apache_translate()
 */
const command_rec edir_apache_cmds[] =
{
	{
		"EnhydraDirectorInherit", cmd_Inherit, NULL, RSRC_CONF, FLAG,
		"Whether default config is inherited by virtual servers."
	},
	{
		"EnhydraDirectorConfigFile", cmd_ConfigFile, NULL, RSRC_CONF, TAKE1,
		"Full path to the Enhydra Director configuration file"
	},
	{
		"EnhydraDirectorDataFile", cmd_DataFile, NULL, RSRC_CONF, TAKE1,
		"Full path to the Enhydra Director scoreboard file"
	},
	{
		"EnhydraDirectorLockFile", cmd_LockFile, NULL, RSRC_CONF, TAKE1,
		"Full path to the Enhydra Director scoreboard lock file"
	},
	{
		"EnhydraDirectorDaemonPath", cmd_DaemonPath, NULL, RSRC_CONF, TAKE1,
		"Full path to the Enhydra Director daemon executable file"
	},
	{
		"EnhydraDirectorDebug", cmd_Debug, NULL, RSRC_CONF, TAKE1,
		"Turn on debug logging."
	},
	{NULL}
};

 