#include <stdio.h>

#include "edir.h"

#include "mod_enhydra_director.h"
#include "edir_apache_log_line.h"

#include "http_log.h"

/**
 * Global reference count array to use instead of scoreboard itself.
 * <P>
 * The scoreboard itself has a reference count for each server entry,
 * but the actual scoreboard access code takes an array of references
 * in order to allow flexible handling of reference counts in multi-
 * process environments such as Apache.
 * <P>
 * It turns out that it is more efficient and less complex to just keep
 * a separate array of reference counts in parallel to the scoreboard.
 * XXX FIX A better solution will be to rearrange the scoreboard to
 * store the reference counts in a contiguous array, as that would also
 * allow non-systemV refcount methods to implement simpler code.
 */
int32 *refCounts = NULL;

/**
 * The number of servers (and thus reference counts).
 */
int32 numServers = 0;

/**
 * Pointer to the global scoreboard.  This scoreboard follows the
 * same format as the ISAPI and Apache modules.  It is position
 * independent, even though it does not need to be.  This allows for
 * maintenance of the scoreboard access code in only one place, which
 * should facilitate increased robustness and reliablility across all
 * platforms.
 */

edir_load_sb_offsets_t *scoreboard = NULL;

/*
 * Use a module-private 'main' pool, with some sub-pools to handle
 * memory allocation and cleanup for various actions.  These pools are:
 * 	edir_apache_pool            -- The main pool.
 * 	edir_apache_pool_request    -- For per-request resources that need
 *                                   to be cleaned up each time.
 * 	edir_apache_pool_current    -- The current pool to use.  This will
 *                                   change, depending upon what's being
 *                                   built or torn down.
 * By doing it this way we can override edir_memory_standard.c with one
 * that uses the 'current' pool so that we can be reasonable sure of
 * memory cleanup when a request is finished.
 */
/* TODO : Proveri da li je ovo nesto sto vazi za sve thread-ove, ili svaki thread mora
da ima svoje promenljive ! S obzirom da su u originalu deklarisane kao
static, verovatno moraju da se definisu za svaki thread posebno ! Pogledaj
kako apache poziva thread-ove, da li poziva i konfiguracionu funkciju
svaki put !*/
pool *edir_apache_pool = NULL;
//pool *edir_apache_pool_request = NULL;
pool *edir_apache_pool_current = NULL;

/*
 * Default config file.
 */
/* TODO : Isti komentar kao i za pool-ove ! */
char * DefaultConfigFile = NULL;


/**
 * Module 'master server' init function.
 * <P>
 * This function is called by Apache in BOTH the (potentially) root-
 * owned 'master' process, and in EACH 'child' process, running as
 * the 'child' user.  The code in this function does anything that
 * may potentially need root privilege, such as creating the
 * '.../logs/edir' directory in which to store the scoreboard shared
 * memory and lock files.
 *
 * On NT it doesn't do much, since Apache/NT uses doesn't need scoreboard
 * shared memory and lock files
 *
 * @param s [in] Server info block for the current (possibly virtual) server.
 * @param p [in] Apache resource pool to use for temporary memory.
 */
void edir_apache_master_init(server_rec *s, pool *p)
{
	ap_add_version_component("Enhydra-Director/4 ported by Rkoma");
}


/**
 * Module initializer callback.
 * <P>
 * This callback gets called to allow the Enhydra Director module to initialize
 * itself for the first time.  We set the log to use the server_rec, and
 * create a module-specific config info block and save it with the server_rec.
 *
 * @param s [in] Server info block for the current (possibly virtual) server.
 * @param p [in] Apache resource pool to use for temporary memory.
 */
void edir_apache_init(server_rec *s, pool *p)
{
	static char			*funcName = "edir_apache_init";
	edir_exception_t	*_exc=NULL, **exc=&_exc;
	edir_apache_cfg_t	*cfg = NULL;
	char				*confDir = NULL, *confFile = NULL;
	edir_conf_t			*appCfg = NULL;	// Conf data for makeing the scoreboard
	int32				sbSize;			// size of a scoreboard
	int					i;
	char				*trace;

	edir_apache_log_set_server_rec(s);

	EDIR_DEBUG_FUNCS_IN(("func: %s("EDIR_PTR", "EDIR_PTR")", funcName, s, p));

	cfg = SERVER_CONFIG(s);

	// Return if already initialized
	if (cfg && cfg->inited) {
		EDIR_DEBUG_VERBOSE(("%s: already initialized", funcName));
		EDIR_DEBUG_FUNCS_OUT(("func: %s() returns void", funcName));
		return;
	}

	if (edir_apache_pool == NULL) {
		edir_apache_pool = ap_make_sub_pool(NULL);
	}
	edir_apache_pool_current = edir_apache_pool;

	/*
	 * Now, try to use 'srm_confname' or 'access_confname' to get the
	 * config files subdirectory.  If neither are set, then just assume
	 * 'conf' under the server root.
	 */
	 /* TODO : Proveri da li Apache/NT API kao directory separator koristi / ili \ !!!!!!! */
	if (s->srm_confname) {
		confDir = ap_server_root_relative(edir_apache_pool, s->srm_confname);
		if (strrchr(confDir, '/')) {
			confDir = ap_pstrdup(p, confDir);
			*(strrchr(confDir, '/')) = '\0'; ;
		} else {
			confDir = ap_server_root_relative(edir_apache_pool, "conf");
			confDir = ap_pstrdup(p, confDir);
		}
	} else if (s->access_confname) {
		confDir = ap_server_root_relative(edir_apache_pool, s->access_confname);
		if (strrchr(confDir, '/')) {
			confDir = ap_pstrdup(p, confDir);
			*(strrchr(confDir, '/')) = '\0'; ;
		} else {
			confDir = ap_server_root_relative(edir_apache_pool, "conf");
			confDir = ap_pstrdup(p, confDir);
		}
	} else {
		/* If we don't know error log, use reasonable default */
#ifdef EDIR_AC_CONFIG_DIR
		confDir = ap_pstrdup(p, EDIR_AC_CONFIG_DIR);
#else
		confDir = ap_server_root_relative(edir_apache_pool, "conf");
		confDir = ap_pstrdup(p, confDir);
#endif
	}

	/*
	 * Generate default config file name.
	 */
	confFile = ap_pstrcat(edir_apache_pool, confDir, "/enhydra_director.conf", NULL);

	/*
	 * Initialize module-specific defaults for the mutex, shmem, and
	 * config implementations we linked with.
	 * No mutex and shmem needed on NT !
	 * Here we use just plain old critical_section !
	 */
	DefaultConfigFile = ap_pstrdup(p, confFile);
	InitializeCriticalSection(&vLock);

	/*
	 * This should be the first attempt at setting up the
	 * scoreboard.
	 */
	/**
	 * Try to open and process the EnhydraLink config file.
	 */
	appCfg = edir_conf_process(confFile, exc);
	if (*exc) EDIR_THROW(*exc);

	/**
	 * Set debugging mask if needed.
	 */
	if (appCfg->debug) edir_debug |= appCfg->debugMask;

	/**
	 * Allocate space for the scoreboard from the global memory
	 * memory heap.  This memory is never freed until the DLL is
	 * unloaded.  Note that we don't use edir_malloc because it's
	 * set to the _temporary_ heap.
	 */
	/* Rkoma's note: edir_malloc IS malloc !!!! */
	sbSize = edir_load_sb_compute_size(appCfg);
	scoreboard = (edir_load_sb_offsets_t *) edir_malloc(sbSize);

	/**
	 * Convert the config data to a position-independent scoreboard.
	 * We don't really need position-independent data here, but we
	 * use it anyhow so that we can use the same scoreboard code
	 * for all module implementations, including Apache.
	 */
	edir_load_sb_from_config(appCfg, (void *)scoreboard);

	numServers = edir_load_sb_num_servers((void *)scoreboard);
	refCounts = (int32 *) edir_malloc(numServers * sizeof(int32));
	for (i=0; i<numServers; i++) refCounts[i] = 0;

	/* debug: print out the scoreboard contents */
	trace = edir_load_sb_debug_string((void *)scoreboard,
	   				refCounts,
					numServers);
	edir_log("Scoreboard:\n%s", trace);
	edir_free(trace);

	EDIR_DEBUG_FUNCS_OUT(("func: %s() returns void", funcName));
	return;

except:
	edir_log_error("Error initializing scoreboard daemon.");
	if (*exc) {
		char *trace;
		trace = edir_exception_stack_trace(*exc);
		edir_log_error(trace);
		edir_free(trace);
		edir_exception_free(exc);
	}
	EDIR_DEBUG_FUNCS_OUT(("func: %s() returns void (exc)", funcName));
	return;
}

/**
 * Process exit callback.
 * <P>
 * Currently we don't do anything here.  It might be nice to try to
 * keep track of the current connection and try to be "nice" if we are
 * killed in the middle, but right now we're mean and nasty and just
 * let the Enhydra server get a plain old IO exception to signal that
 * the connection dropped.
 *
 * @param s [in] Server info block for the current (possibly virtual) server.
 * @param p [in] Apache resource pool to use for temporary memory.
 */
void edir_apache_child_exit(server_rec *s, pool *p)
{
	DeleteCriticalSection(&vLock);

	edir_apache_log_set_server_rec(s);
	EDIR_DEBUG_FUNCS_IN(("func: edir_apache_child_exit("EDIR_PTR", "EDIR_PTR")", s, p));
	EDIR_DEBUG_FUNCS_OUT(("func: edir_apache_child_exit() returns void"));
}


/**
 * Create a per-directory configuration record for mod_enhydra_director.
 * <P>
 * Note that this won't be called, since Enhydra Director is not configurable
 * on a per-directory basis.
 *
 * @param p       [in,out] Apache resource pool for temporarily allocated data.
 * @param dirspec [in      Directory under which this configuration resides.
 * @return The configuration record.
 */
void *edir_apache_create_dir_config(pool *p, char *dirspec)
{
	edir_apache_cfg_t *cfg;
	cfg = (edir_apache_cfg_t *) ap_pcalloc(p, sizeof(edir_apache_cfg_t));
	cfg->cfg_Mode = EDIR_APACHE_CFG_MODE_DIR;
	cfg->inited = 0;
	cfg->cfg_Inheritable = 0;
	cfg->cfg_Server = NULL;
	cfg->cfg_Dir = dirspec ? ap_pstrdup(p, dirspec) : NULL;
	cfg->configFile = NULL;
	cfg->inBufSize = EDIR_APACHE_INBUF_SIZE;
	cfg->outBufSize = EDIR_APACHE_OUTBUF_SIZE;
	cfg->sessionIdPrefix = ap_pstrdup(p, EDIR_LOAD_DEFAULT_SID_PREFIX);
	return (void *)cfg;
}

/**
 * Create a per-directory configuration record for mod_enhydra_director.
 * <P>
 * Note that this won't be called, since Enhydra Director is not configurable
 * on a per-directory basis.
 *
 * @param p       [in,out] Apache resource pool for temporarily allocated data.
 * @param dirspec [in      Directory under which this configuration resides.
 * @return The configuration record.
 */
void *edir_apache_create_server_config(pool *p, server_rec *s)
{
	edir_apache_cfg_t	*cfg;
	cfg = (edir_apache_cfg_t *) ap_pcalloc(p, sizeof(edir_apache_cfg_t));
	cfg->cfg_Mode = EDIR_APACHE_CFG_MODE_SERVER;
	cfg->inited = 0;
	cfg->cfg_Inheritable = 0;
	if (s->server_hostname)
	cfg->cfg_Server = ap_pstrdup(p, s->server_hostname);
	else
	cfg->cfg_Server = NULL;
	cfg->cfg_Dir = NULL;
	cfg->configFile = NULL;
	cfg->inBufSize = EDIR_CLIENT_DEFAULT_INBUF_SIZE;
	cfg->outBufSize = EDIR_CLIENT_DEFAULT_OUTBUF_SIZE;
	cfg->sessionIdPrefix = ap_pstrdup(p, EDIR_LOAD_DEFAULT_SID_PREFIX);
	return (void *)cfg;
}


