/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_client.c,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */

#include "edir.h"

extern void
edir_client_connect(edir_connection_t *conn,
		  struct sockaddr_in *localAddr,
		  struct sockaddr_in *serverAddr,
		  char  *authData,
		  octet *sendBuf, int32 sendBufMax,
		  octet *receiveBuf, int32 receiveBufMax,
		  edir_exception_t **exc)
{
    static char *funcName = "edir_client_connect";
    /* Make sure we're doing this in the right connection state. */
    switch (conn->state) {
	case EDIR_STATE_CLOSED:
	    break;
	default:
	    *exc = edir_exception_create(EDIR_EXCEPT_PROTO_BAD_STATE,
			"edir_client_connect: Must be called in state %s%s",
			"CLOSED, not ", edir_statename(conn->state));
	    EDIR_THROW(*exc);
    }

    /*
     * Set up connection parameters.
     */
    if (localAddr == NULL) {
#ifdef INADDR_ANY
	conn->localAddr.sin_addr.s_addr = INADDR_ANY;
	conn->localAddr.sin_port = 0;
#else
	memset((char *) &(conn->localAddr), 0, sizeof(struct sockaddr_in));
#endif
    } else {
	memcpy((char *) &(conn->localAddr), (char *)localAddr,
	       sizeof(struct sockaddr_in));
    }
    if (serverAddr == NULL) {
#ifdef INADDR_ANY
	conn->serverAddr.sin_addr.s_addr = INADDR_ANY;
	conn->serverAddr.sin_port = 0;
#else
	memset((char *) &(conn->serverAddr), 0, sizeof(struct sockaddr_in));
#endif
    } else {
	memcpy((char *) &(conn->serverAddr), (char *)serverAddr,
	       sizeof(struct sockaddr_in));
    }
    strncpy(conn->authData, authData, EDIR_AUTH_MAX);
    conn->authData[EDIR_AUTH_MAX] = '\0';
    conn->serverMajorVersion = 0;
    conn->serverMinorVersion = 0;
    conn->receiveData = receiveBuf;
    conn->receivePtr = receiveBuf;
    conn->receiveDataLen = 0;
    conn->receiveDataMax = receiveBufMax;
    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = 0;
    conn->sendDataMax = sendBufMax;
    conn->haveHeader = 0;
    conn->haveData = 0;
    conn->ioPending = 0;
    conn->sbuf = edir_sbuf_create();
    
    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

extern void
edir_client_send_header(edir_connection_t *conn,
		      void *headerName, int32 headerNameLen,
		      void *headerValue, int32 headerValueLen, int16 subType,
		      octet *sendBuf, int32 sendBufMax,
		      octet *receiveBuf, int32 receiveBufMax,
		      edir_exception_t **exc)
{
    static char *funcName = "edir_client_send_header";

    /* Make sure we're doing this in the right connection state. */
    switch (conn->state) {
	case EDIR_STATE_OPEN:
	    conn->state = EDIR_STATE_SEND_HEADERS;
	    break;
	case EDIR_STATE_SEND_HEADERS:
	    break;
	default:
	    *exc = edir_exception_create(EDIR_EXCEPT_PROTO_BAD_STATE,
		    "Function %s may only be called in state %s%s", funcName,
		    "OPEN or SEND_HEADERS, not ", edir_statename(conn->state));
	    EDIR_THROW(*exc);
    }

    /*
     * Set outgoing header data.  The data buffers are passed by reference, 
     * and NOT copied!  Callers need to be mindful of this.
     */
    conn->headerName = headerName;
    conn->headerNameLen = headerNameLen;
    conn->headerValue = headerValue;
    conn->headerValueLen = headerValueLen;
    conn->isLast = (subType == EDIR_PKT_SUBTYPE_HEADER_LAST);
    conn->receiveData = receiveBuf;
    conn->receivePtr = receiveBuf;
    conn->receiveDataLen = 0;
    conn->receiveDataMax = receiveBufMax;
    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = 0;
    conn->sendDataMax = sendBufMax;
    conn->haveHeader = 0;
    conn->haveData = 0;
    conn->packetType = EDIR_PKT_TYPE_HEADER;
    conn->packetSubType = (octet) subType;
    conn->state = EDIR_STATE_SEND_HEADER;

    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

extern void
edir_client_send_data(edir_connection_t *conn, int32 nbytes, int16 isLast,
		    octet *sendBuf, int32 sendBufMax,
		    octet *receiveBuf, int32 receiveBufMax,
		    edir_exception_t **exc)
{
    static char *funcName = "edir_client_send_data";
    /* Make sure we're doing this in the right connection state. */
    switch (conn->state) {
	case EDIR_STATE_HEADERS_SENT:
	    conn->state = EDIR_STATE_SEND_ENTITY;
	    break;
	case EDIR_STATE_SEND_ENTITY:
	    break;
	default:
	    *exc = edir_exception_create(EDIR_EXCEPT_PROTO_BAD_STATE,
		    "Function %s may only be called in state %s%s", funcName,
		    "HEADERS_SENT or SEND_ENTITY, not ",
		    edir_statename(conn->state));
	    EDIR_THROW(*exc);
    }

    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = nbytes;
    conn->sendDataMax = sendBufMax;
    conn->receiveData = receiveBuf;
    conn->receivePtr = receiveBuf;
    conn->receiveDataLen = 0;
    conn->receiveDataMax = receiveBufMax;
    conn->isLast = (octet) isLast;
    conn->haveHeader = 0;
    conn->haveData = 0;

    conn->state = EDIR_STATE_SEND_DATA;

    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

extern void
edir_client_next_header(edir_connection_t *conn, 
		    octet *sendBuf, int32 sendBufMax,
		    octet *receiveBuf, int32 receiveBufMax,
		    edir_exception_t **exc)
{
    static char *funcName = "edir_client_next_header";
    /* Make sure we're doing this in the right connection state. */
    switch (conn->state) {
	case EDIR_STATE_RECEIVE_HEADERS:
	    break;
	default:
	    *exc = edir_exception_create(EDIR_EXCEPT_PROTO_BAD_STATE,
		    "Function %s may only be called in state %s%s", funcName,
		    "EDIR_STATE_RECEIVE_HEADERS, not ",
		    edir_statename(conn->state));
	    EDIR_THROW(*exc);
    }

    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = 0;
    conn->sendDataMax = sendBufMax;
    conn->receiveData = receiveBuf;
    conn->receivePtr = receiveBuf;
    conn->receiveDataLen = 0;
    conn->receiveDataMax = receiveBufMax;
    conn->haveHeader = 0;
    conn->haveData = 0;

    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

extern void
edir_client_next_data(edir_connection_t *conn, 
		    octet *sendBuf, int32 sendBufMax,
		    octet *receiveBuf, int32 receiveBufMax,
		    edir_exception_t **exc)
{
    static char *funcName = "edir_client_next_data";
    /* Make sure we're doing this in the right connection state. */
    switch (conn->state) {
	case EDIR_STATE_RECEIVE_RESPONSE:
	case EDIR_STATE_RECEIVE_RESPONSE_PACKET:
	    break;
	default:
	    *exc = edir_exception_create(EDIR_EXCEPT_PROTO_BAD_STATE,
		    "Function %s may only be called in state %s%s", funcName,
		    "RECEIVE_RESPONSE or RECEIVE_RESPONSE_PACKET, not ",
		    edir_statename(conn->state));
	    EDIR_THROW(*exc);
    }

    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = 0;
    conn->sendDataMax = sendBufMax;
    conn->receiveData = receiveBuf;
    conn->receivePtr = receiveBuf;
    conn->receiveDataLen = 0;
    conn->receiveDataMax = receiveBufMax;
    conn->haveHeader = 0;
    conn->haveData = 0;

    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

extern void
edir_client_call_function(edir_connection_t *conn,
			octet funcType, int oneWay, int isLast, int32 argLen,
			octet *sendBuf, int32 sendBufMax,
			octet *receiveBuf, int32 receiveBufMax,
			edir_exception_t **exc)
{
    static char *funcName = "edir_client_call_function";

    /* Make sure we're doing this in the right connection state. */
    switch (conn->state) {
	case EDIR_STATE_OPEN:
	case EDIR_STATE_SEND_HEADERS:
	case EDIR_STATE_HEADERS_SENT:
	case EDIR_STATE_SEND_ENTITY:
	    /* Want to return to current state after func. */
	    conn->returnState = conn->state;
	    conn->state = EDIR_STATE_SEND_FUNC;
	    break;
	default:
	    *exc = edir_exception_create(EDIR_EXCEPT_PROTO_BAD_STATE,
		    "Function %s may only be called in state %s%s", funcName,
		    "OPEN, SEND_HEADERS, HEADERS_SENT or SEND_ENTITY; not ",
		    edir_statename(conn->state));
	    EDIR_THROW(*exc);
    }

    conn->funcType = funcType;
    conn->funcOneWay = oneWay;
    conn->isLast = isLast;
    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = argLen;
    conn->sendDataMax = sendBufMax;
    conn->receiveData = receiveBuf;
    conn->receivePtr = receiveBuf;
    conn->receiveDataLen = 0;
    conn->receiveDataMax = receiveBufMax;

    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

/**
 * Perform a graceful close on this connection.
 * <P> 
 * This can ONLY be done in state OPEN.  To close a connection
 * in other states, use "edir_client_abort()".
 * 
 * @param conn  [in,out] Connection context to operate on.
 * @param exc   [in,out] Exception context to catch errors.
 */
extern void
edir_client_close(edir_connection_t *conn,
		octet *sendBuf, int32 sendBufMax,
		edir_exception_t **exc)
{
    static char *funcName = "edir_client_close";

    /* Make sure we're doing this in the right connection state. */
    switch (conn->state) {
	case EDIR_STATE_OPEN:
	    break;
	default:
	    *exc = edir_exception_create(EDIR_EXCEPT_PROTO_BAD_STATE,
		    "Function %s may only be called in state %s%s", funcName,
		    "OPEN; not ", edir_statename(conn->state));
	    EDIR_THROW(*exc);
    }

    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = 0;
    conn->sendDataMax = sendBufMax;
    conn->receiveData = conn->receivePtr = NULL;
    conn->receiveDataLen = conn->receiveDataMax = 0;
    conn->fatalException = NULL; /* Required - fixed nasty bug */

    conn->state = EDIR_STATE_CLOSE;

    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

/**
 * Abort the connection in as graceful manner as possible.
 * <P>
 * This function can be called in any state.  It will make an attempt
 * to send EDIR_PKT_SUBTYPE_PROTO_FATAL to the server with the
 * message contained in "excIn".  After this call, the connection
 * is guaranteed to be in a fully CLOSED state.  Any exceptions
 * returned by this function can probably be discarded, but are
 * returned anyhow for debugging and other uses.
 * 
 * @param conn  [in,out] Connection context to operate on.
 * @param exc   [in,out] Exception context to catch errors.
 */
extern void
edir_client_abort(edir_connection_t *conn,
		octet *sendBuf, int32 sendBufMax,
		edir_exception_t **exc)
{
    static char *funcName = "edir_client_abort";

    conn->sendData = sendBuf;
    conn->sendPtr = sendBuf;
    conn->sendDataLen = 0;
    conn->sendDataMax = sendBufMax;
    conn->receiveData = conn->receivePtr = NULL;
    conn->receiveDataLen = conn->receiveDataMax = 0;
    conn->fatalException = *exc;
    conn->state = EDIR_STATE_ABORT;
    if (*exc) {
	conn->fatalException = *exc;
	*exc = NULL;
    }

    /* Let state machine do its work. */
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

/**
 * See if the pending non-blocking or event-driven I/O is finished.
 * <P>
 * If one of the other edir_client_*() functions returns and 
 * EDIR_IO_PENDING() returns true on the connection, then I/O is not
 * comleted for that call, and the calling code must exit to the
 * event dispatcher (usually a select() loop somewhere).  When
 * the status of the I/O operation changes, this function is used
 * to process any newly received status and/or data on the connection.
 * <P>
 * The I/O operation should be considered "in progress" until and unless
 * this or the original "edir_client" function returns and EDIR_IO_PENDING()
 * returns false on the connection.
 * 
 * @param conn [in,out]	Connection context in the midst of non-blocked I/O.
 * @param exc  [out]	Exception context to catch errors.
 */
extern void
edir_client_io_event(edir_connection_t *conn, edir_exception_t **exc)
{
    static char *funcName = "edir_client_io_event";
    edir_state_machine(conn, exc);
    if (*exc) EDIR_THROW(*exc);
    return;

except:
    return;
}

