/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_conf.c,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */
#include "edir.h"

/**
 * Create an empty <code>edir_conf_appserver_t</code> structure and initialize
 * its fields to default values.
 * 
 * @return A pointer to the newly initialized structure.
 */
extern edir_conf_appserver_t *
edir_conf_appserver_create()
{
    edir_conf_appserver_t *appServer;
    appServer = (edir_conf_appserver_t *)
		edir_malloc(sizeof(edir_conf_appserver_t));
    memset((char *)appServer, 0, sizeof(edir_conf_appserver_t));
    appServer->weight = 1;
    appServer->weightIsDefault = 1;
    return appServer;
}

/**
 * Free a <code>edir_conf_appserver_t</code> structure and all of its child
 * structures.
 * 
 * @param server [in,out] Pointer to the structure to free.
 */
extern void
edir_conf_appserver_free(edir_conf_appserver_t *server)
{
    if (server) {
	if (server->hostName) edir_free(server->hostName);
	if (server->ipName) edir_free(server->ipName);
	if (server->authData) edir_free(server->authData);
	memset((char *)server, 0, sizeof(edir_conf_appserver_t));
	edir_free(server);
    }
}

/**
 * Create an empty <code>edir_conf_appserver_t</code> array with the specified
 * number of initially NULL elements.
 * 
 * @return A pointer to the newly initialized array.
 */
extern edir_conf_appserver_t **
edir_conf_appservers_create(int32 num)
{
    edir_conf_appserver_t **servers;
    servers = (edir_conf_appserver_t **)
	      edir_malloc((num + 1) * sizeof(edir_conf_appserver_t *));
    memset((char *)servers, 0, (num + 1) * sizeof(edir_conf_appserver_t *));
    return servers;
}

/**
 * Free a <code>edir_conf_appserver_t</code> array and all of its child
 * structures.
 * 
 * @param servers [in,out] Pointer to the array to free.
 */
extern void
edir_conf_appservers_free(edir_conf_appserver_t **servers)
{
    edir_conf_appserver_t **spp;
    if (servers) {
	for (spp=servers; *spp; spp++) {
	    edir_conf_appserver_free(*spp);
	    *spp = NULL;
	}
	edir_free(servers);
    }
}

/**
 * Create an empty <code>edir_conf_restrict_t</code> structure and initialize
 * its fields to default values.
 * 
 * @return A pointer to the newly initialized structure.
 */
extern edir_conf_restrict_t *
edir_conf_restrict_create()
{
    edir_conf_restrict_t *restrict_ent;
    restrict_ent = (edir_conf_restrict_t *)
	       edir_malloc(sizeof(edir_conf_restrict_t));
    memset((char *)restrict_ent, 0, sizeof(edir_conf_restrict_t));
    return restrict_ent;
}

/**
 * Free a <code>edir_conf_restrict_t</code> structure and all of its child
 * structures.
 * 
 * @param restrict [in,out] Pointer to the structure to free.
 */
extern void
edir_conf_restrict_free(edir_conf_restrict_t *restrict_ent)
{
    if (restrict_ent) {
	if (restrict_ent->hostName) edir_free(restrict_ent->hostName);
	memset((char *)restrict_ent, 0, sizeof(edir_conf_restrict_t));
	edir_free(restrict_ent);
    }
}

/**
 * Create an empty <code>edir_conf_restrict_t</code> array with the specified
 * number of initially NULL elements.
 * 
 * @return A pointer to the newly initialized array.
 */
extern edir_conf_restrict_t **
edir_conf_restricts_create(int32 num)
{
    edir_conf_restrict_t **restricts;
    restricts = (edir_conf_restrict_t **)
	        edir_malloc((num + 1) * sizeof(edir_conf_restrict_t *));
    memset((char *)restricts, 0, (num + 1) * sizeof(edir_conf_restrict_t *));
    return restricts;
}

/**
 * Free a <code>edir_conf_restrict_t</code> array and all of its child
 * structures.
 * 
 * @param restricts [in,out] Pointer to the array to free.
 */
extern void
edir_conf_restricts_free(edir_conf_restrict_t **restricts)
{
    edir_conf_restrict_t **rpp;
    if (restricts) {
	for (rpp=restricts; *rpp; rpp++) {
	    edir_conf_restrict_free(*rpp);
	    *rpp = NULL;
	}
	edir_free(restricts);
    }
}

/**
 * Create an empty <code>edir_conf_app_t</code> structure and initialize its
 * fields to default values.
 * 
 * @return A pointer to the newly initialized structure.
 */
extern edir_conf_app_t *
edir_conf_app_create()
{
    edir_conf_app_t *app;
    app = (edir_conf_app_t *) edir_malloc(sizeof(edir_conf_app_t));
    memset((char *)app, 0, sizeof(edir_conf_app_t));
    return app;
}

/**
 * Free a <code>edir_conf_app_t</code> structure and all of its child
 * structures.
 * 
 * @param app [in,out] Pointer to the structure to free.
 */
extern void
edir_conf_app_free(edir_conf_app_t *app)
{
    if (app) {
	if (app->appPrefix) edir_free(app->appPrefix);
	if (app->servers) edir_conf_appservers_free(app->servers);
	if (app->restricts) edir_conf_restricts_free(app->restricts);
	memset((char *)app, 0, sizeof(edir_conf_app_t));
    }
}

/**
 * Create an empty <code>edir_conf_app_t</code> array with the specified
 * number of initially NULL elements.
 * 
 * @return A pointer to the newly initialized array.
 */
extern edir_conf_app_t **
edir_conf_apps_create(int32 num)
{
    edir_conf_app_t **apps;
    apps = (edir_conf_app_t **)
	    edir_malloc((num + 1) * sizeof(edir_conf_app_t *));
    memset((char *)apps, 0, (num + 1) * sizeof(edir_conf_app_t *));
    return apps;
}

/**
 * Free a <code>edir_conf_app_t</code> array and all of its child
 * structures.
 * 
 * @param apps [in,out] Pointer to the array to free.
 */
extern void
edir_conf_apps_free(edir_conf_app_t **apps)
{
    edir_conf_app_t **app;
    if (apps) {
	for (app=apps; *app; app++) {
	    edir_conf_app_free(*app);
	    *app = NULL;
	}
	edir_free(apps);
    }
}

/**
 * Create an empty <code>edir_conf_stat_t</code> structure and initialize its
 * fields to default values.
 * 
 * @return A pointer to the newly initialized structure.
 */
extern edir_conf_stat_t *
edir_conf_stat_create()
{
    edir_conf_stat_t *stat;
    stat = (edir_conf_stat_t *) edir_malloc(sizeof(edir_conf_stat_t));
    memset((char *)stat, 0, sizeof(edir_conf_stat_t));
    return stat;
}

/**
 * Free a <code>edir_conf_stat_t</code> structure and all of its child
 * structures.
 * 
 * @param stat [in,out] Pointer to the structure to free.
 */
extern void
edir_conf_stat_free(edir_conf_stat_t *stat)
{
    if (stat) {
	if (stat->statPrefix) edir_free(stat->statPrefix);
	if (stat->restricts) edir_conf_restricts_free(stat->restricts);
	memset((char *)stat, 0, sizeof(edir_conf_stat_t));
    }
}

/**
 * Create an empty <code>edir_conf_stats_t</code> array with the specified
 * number of initially NULL elements.
 * 
 * @return A pointer to the newly initialized array.
 */
extern edir_conf_stat_t **
edir_conf_stats_create(int32 num)
{
    edir_conf_stat_t **stats;
    stats = (edir_conf_stat_t **)
	    edir_malloc((num + 1) * sizeof(edir_conf_stat_t *));
    memset((char *)stats, 0, (num + 1) * sizeof(edir_conf_stat_t *));
    return stats;
}

/**
 * Free a <code>edir_conf_stat_t</code> array and all of its child
 * structures.
 * 
 * @param stats [in,out] Pointer to the array to free.
 */
extern void
edir_conf_stats_free(edir_conf_stat_t **stats)
{
    edir_conf_stat_t **stat;
    if (stats) {
	for (stat=stats; *stat; stat++) {
	    edir_conf_stat_free(*stat);
	    *stat = NULL;
	}
	edir_free(stats);
    }
}

/**
 * Create an empty <code>edir_conf_t</code> structure and initialize its
 * default member values.
 * 
 * @return A pointer to the newly initialized structure.
 */
extern edir_conf_t *
edir_conf_create()
{
    edir_conf_t *config;
    config = (edir_conf_t *) edir_malloc(sizeof(edir_conf_t));
    memset((char *)config, 0, sizeof(edir_conf_t));
    config->daemonTimeout = -1;
    config->daemonInterval = -1;
    return config;
}

/**
 * Free a <code>edir_conf_t</code> structure and all of its child
 * structures.
 * 
 * @param config [in,out] Pointer to the structure to free.
 */
extern void
edir_conf_free(edir_conf_t *config)
{
    if (config->daemonProg) edir_free(config->daemonProg);
    if (config->apps) edir_conf_apps_free(config->apps);
    if (config->stats) edir_conf_stats_free(config->stats);
    memset((char *)config, 0, sizeof(edir_conf_t));
}

/**
 * Read a text configuration file into an argv-style line array.
 * <P>
 * The returned array is an array of character pointers terminated
 * by a NULL character pointer.
 * 
 * @param fileName [in]     The name of the file to read.
 * @param exc      [in,out] Exception handle filled in if an error occurs.
 * @return An argv-style NULL-terminated array of character pointers.
 * @exception EDIR_EXCEPT_IO_ERROR If the file cannot be opened or if
 * 	an error occurs while reading the file.
 */
extern char **
edir_conf_read_file_lines(char *fileName, edir_exception_t **exc)
{
    static char *funcName = "edir_conf_read_file_lines";
    char lineBuf[EDIR_CONF_MAX_LINE_LEN+1];
    char **lines = NULL;
    char **tmplines = NULL;
    char *cp;
    int32 maxLines = 0;
    int32 lineNumber = 0; /* This starts at zero, NOT one! */
    FILE *fp = NULL;

    fp = fopen(fileName, "r");
    if (fp == NULL) {
	*exc = edir_exception_create(EDIR_EXCEPT_IO_ERROR,
				   "Cannot open config file '%s'", fileName);
	EDIR_THROW(*exc);
    }

    while (fgets(lineBuf, EDIR_CONF_MAX_LINE_LEN, fp) != NULL) {
	if ((cp = strchr(lineBuf, '\n')) != NULL) *cp = '\0';
	if ((cp = strchr(lineBuf, '\r')) != NULL) *cp = '\0';
	if (lineNumber >= maxLines) {
	    tmplines = (char **) edir_malloc(
		(maxLines + EDIR_CONF_MAXLINES_INCR + 1) * sizeof(char *));
	    memset(tmplines, 0, 
		(maxLines + EDIR_CONF_MAXLINES_INCR + 1) * sizeof(char *));
	    if (lines != NULL) {
		memcpy(tmplines, lines, maxLines * sizeof(char *));
		edir_free(lines);
	    }
	    lines = tmplines;
	    maxLines += EDIR_CONF_MAXLINES_INCR;
	}
	lines[lineNumber++] = edir_strdup(lineBuf);
    }
    if (ferror(fp)) {
	*exc = edir_exception_create(EDIR_EXCEPT_IO_ERROR,
		   "Read error at line %d in config file '%s'",
		    lineNumber + 1, fileName);
	EDIR_THROW(*exc);
    }
    fclose(fp);
    return lines;

except:
    /* Clean up and free allocated strings. */
    if (lines) {
	char **cpp;
	for (cpp = lines; *cpp; cpp++) {
	    edir_free(*cpp);
	    *cpp = NULL;
	}
	edir_free(lines);
    }
    /* Close config file if open */
    if (fp != NULL) fclose(fp);
    return NULL;
}

/**
 * Free an argv-style list of file line buffers.
 * <P>
 * @param lines [in,out] The line buffer list to free.
 */
extern void
edir_conf_free_file_lines(char **lines)
{
    if (lines) {
	while (*lines) {
	    edir_free(*lines);
	    *lines++ = NULL;
	}
	edir_free(lines);
    }
}

