/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_conf.h,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */

#ifndef __EDIR_CONF_H__
#define __EDIR_CONF_H__ 1
#ifdef __cplusplus
extern "C" {
#endif

#define EDIR_CONF_MAXLINES_INCR		256
#define EDIR_CONF_MAX_LINE_LEN		2048

#define	EDIR_CONF_DEFAULT_FAIL_RETRY	30

/**
 * Data structures related to the Enhydra Director ad-hoc configuration
 * file functions.
 * <P>
 * XXX FIX - This should go away in favor of an XML based config file.
 */

/**
 * Temporary configuration record for an application server.
 * <P>
 * Normally these records are converted into a scoreboard file.
 */
typedef struct edir_conf_appserver {
    char		*hostName;
    char		*ipName;
    uint16		port;
    struct sockaddr_in	sin;
    int32		weight;
    int			weightIsDefault;
    char		*authData;
} edir_conf_appserver_t;

/**
 * Server host restriction record.
 * <P>
 * These records represent 'restrictions' on the scope of an
 * application config to a particular virtual host name or IP
 * address and/or port number.
 */
typedef struct edir_conf_restrict {
    int			type;
    char		*hostName;
    uint16		port;
    struct sockaddr_in	sin;
    int32		netBits;
    uint32		netMask;
} edir_conf_restrict_t;
#define EDIR_CONF_RESTRICT_SERVER	1
#define EDIR_CONF_RESTRICT_VIRTUAL	2
#define EDIR_CONF_RESTRICT_CLIENT	3

/**
 * Per-application temporary configuration record.
 */
typedef struct edir_conf_app {
    char		*appPrefix;
    edir_conf_appserver_t **servers;
    edir_conf_restrict_t **restricts;
} edir_conf_app_t;

/**
 * Statistics port configuration.
 */
typedef struct edir_conf_stat {
    char		 *statPrefix;
    edir_conf_restrict_t **restricts;
} edir_conf_stat_t;

/**
 * General configuration data header.
 */
typedef struct edir_conf {
    char		*daemonProg;
    int32		daemonTimeout;
    int32		daemonInterval;
    int32		daemonDebug;	/* Whether daemon should debug */
    int32		daemonDebugMask;/* Daemon's debugging mask */
    int32		debug;		/* Whether module/utils debug */
    int32		debugMask;	/* Module/util Debugging mask. */
    int32		failRetry;
    int32		failRetrySet;
    edir_conf_app_t	**apps;
    edir_conf_stat_t	**stats;
} edir_conf_t;

/**
 * Process configuration info from the named source.
 * <P>
 * Currently only the file data source (edir_conf_file.c) is supported.  
 * In the future, we should be able to read configurations from
 * other sources, such as an Enhydra config server, etc.
 * 
 * @param configName [in] String name identifying the config data source.
 * @param exc [out] Exception pointer set if a configuration error occurs.
 * @return A pointer to an "edir_conf_t" structure containing the configuration.
 * @exception EDIR_EXCEPT_CONFIG If a configuration error occurs.
 */
extern edir_conf_t *
edir_conf_process(char *configName, edir_exception_t **exc);

/**
 * Allow specific modules to set their own default file name.
 * <P>
 * This file name may be ignored, if the underlying mechanism does
 * not use files.
 * 
 * @param filename [in] The new default file name.
 */
extern void
edir_conf_set_default_filename(char  *filename);

/*
 * Common config tree functions
 */
extern edir_conf_appserver_t * edir_conf_appserver_create();
extern void edir_conf_appserver_free(edir_conf_appserver_t *server);
extern edir_conf_appserver_t ** edir_conf_appservers_create(int32 num);
extern void edir_conf_appservers_free(edir_conf_appserver_t **servers);
extern edir_conf_restrict_t * edir_conf_restrict_create();
extern void edir_conf_restrict_free(edir_conf_restrict_t *restrict);
extern edir_conf_restrict_t ** edir_conf_restricts_create(int32 num);
extern void edir_conf_restricts_free(edir_conf_restrict_t **restricts);
extern edir_conf_app_t * edir_conf_app_create();
extern void edir_conf_app_free(edir_conf_app_t *app);
extern edir_conf_app_t ** edir_conf_apps_create(int32 num);
extern void edir_conf_apps_free(edir_conf_app_t **apps);
extern edir_conf_stat_t * edir_conf_stat_create();
extern void edir_conf_stat_free(edir_conf_stat_t *stat);
extern edir_conf_stat_t ** edir_conf_stats_create(int32 num);
extern void edir_conf_stats_free(edir_conf_stat_t **stats);
extern edir_conf_t * edir_conf_create();
extern void edir_conf_free(edir_conf_t *config);
extern char **edir_conf_read_file_lines(char *fileName,edir_exception_t **exc);
extern void edir_conf_free_file_lines(char **lines);

#ifdef __cplusplus
} /* Extern "C" */
#endif
#endif /* __EDIR_CONF_H__ */

