/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_cookie_b64.c,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */

#ifndef GEN_DECODE_TABLE
#include "edir.h"
#else
typedef unsigned char octet;
typedef unsigned long uint32;
typedef unsigned short uint16;
#include "edir_cookie_b64.h"
#endif

/**
 * Modified Base-64 encoding table.
 * Not RFC compliant, but compatible with Enhydra session cookie
 * encodings, which must be safe in URLs.  We replace '+' with '_'
 * and '/' with '-'.  If this table is changed, compile this file
 * with -DGEN_DECODE_TABLE, run the executable to get a new 
 * decodings table, and replace the decodings table below with
 * the newly generated one.
 */
static octet encodings[64] = {
    /*00-07*/  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 
    /*08-15*/  'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 
    /*16-23*/  'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 
    /*24-31*/  'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f', 
    /*32-39*/  'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 
    /*40-47*/  'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 
    /*48-55*/  'w', 'x', 'y', 'z', '0', '1', '2', '3', 
    /*56-63*/  '4', '5', '6', '7', '8', '9', '_', '-'
};

/**
 * Table to decode ASCII characters to the correct 6 bit value.
 * <P>
 * If the 'encodings' table above is changed, this table should
 * be automatically regenerated by compiling this module with 
 * the GEN_DECODE_TABLE macro defined and running the resulting
 * executable.
 * <P>
 * Note: This table was generated for the Enhydra modified base 64, but
 * it was also hand edited to correctly decode 'standard' base 64 as well.
 */
static char decodings[256] = {
    /*  0*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /* 16*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /* 32*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, 63, -1, 63,
    /* 48*/  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1,
    /* 64*/  -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
    /* 80*/  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, 62,
    /* 96*/  -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
    /*112*/  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
    /*128*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /*144*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /*160*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /*176*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /*192*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /*208*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /*224*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    /*240*/  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

/**
 * Encode a byte array of length 'inLen' into a base64 encoded string.
 * <P>
 * The output buffer must be large enough to hold the encoded string.
 * The 'EDIR_COOKIE_B64_ENC_SIZE(N)' macro can be used to determine the
 * exact number of characters needed to encode 'N' bytes of data.  The
 * length of the output buffer should allow for an extra terminating
 * null character, thus for an array of N bytes, pass an output buffer
 * of 'EDIR_COOKIE_B64_ENC_SIZE(N) + 1' bytes.
 * 
 * @param inBuf  [in]  The input byte array.
 * @param inLen  [in]  The length in bytes of the input array.
 * @param outBuf [out] Pointer to the array of characters to write to.
 * @return The length of the encoded string.
 */
extern int
edir_cookie_b64_encode(register octet *inBuf, register int inLen,
		     register char *outBuf)
{
    int		ret  = EDIR_COOKIE_B64_ENC_SIZE(inLen);
    register	uint32	ival;
    for (; inLen >= 3; inBuf += 3, outBuf += 4, inLen -= 3) {
	ival =  inBuf[0] << 16;
	ival |= inBuf[1] << 8;
	ival |= inBuf[2];
	outBuf[3] = encodings[ival & 63]; ival >>= 6;
	outBuf[2] = encodings[ival & 63]; ival >>= 6;
	outBuf[1] = encodings[ival & 63]; ival >>= 6;
	outBuf[0] = encodings[ival & 63];
    }
    switch (inLen) {
	case 0:
	    outBuf[0] = '\0';
	    break;
	case 1:
	    outBuf[4] = '\0';
	    outBuf[3] = outBuf[2] = '=';
	    ival = inBuf[0] << 4;
	    outBuf[1] = encodings[ival & 63]; ival >>= 6;
	    outBuf[0] = encodings[ival & 63];
	    break;
	case 2:
	    outBuf[4] = '\0';
	    outBuf[3] = '=';
	    ival = (inBuf[0] << 10) + (inBuf[1] << 2);
	    outBuf[2] = encodings[ival & 63]; ival >>= 6;
	    outBuf[1] = encodings[ival & 63]; ival >>= 6;
	    outBuf[0] = encodings[ival & 63];
	    break;
    }
    return ret;
}

/**
 * Decode a base64 encoded string into its original binary form.
 * <P>
 * This function skips over characters not listed in the encodings 
 * table, and may not process all of the input characters if
 * insufficient characters are presented to decode a complete
 * block of 3 output bytes. An '=' character always terminates
 * the decoding.
 * <P>
 * The input buffer must be a null terminated string, and the
 * output buffer must be sufficiently large to hold the
 * decoded bytes.  As a convenience, the EDIR_COOKIE_B64_DEC_SIZE(N)
 * macro may be used to compute the minimum number of bytes 
 * needed to insure against buffer overflow.
 * <P>
 * Since input characters may be skipped or left unread due
 * to premature end of string, this function may not decode all
 * input bytes.  The number of input bytes actually decoded is
 * passed via the integer pointer 'inRead' if it is not NULL.
 * The number of bytes actually decoded is returned by the function.
 * 
 * @param inBuf  [in]     The string to decode.
 * @param outBuf [out]    Buffer to receive the decoded bytes.
 * @param inRead [in,out] Optional int pointer to receive input bytes
 * 			  processed.
 * @return The number of decoded bytes written to 'outBuf'.
 */
extern int
edir_cookie_b64_decode(char *inBuf, octet *outBuf, int *inRead)
{
    char		tbuf[4];
    register char	*iptr = inBuf, *mark;
    register octet	*optr = outBuf;
    register int	n, eq;
    register uint32	ival;
    for (;;) {
	/* Gather next 4 legal characters */
	for (n=0, eq=0, mark=iptr; (*iptr) && (n < 4); iptr++) {
	    if ((tbuf[n] = decodings[*((octet *)iptr)]) >= 0) n++;
	    else if (*iptr == '=') {
		/* Eat all '=' terminators */
		iptr++;
		while (*iptr == '=') iptr++;
		eq = 1;
		break;
	    }
	}
	switch (n) {
	    case 4:
		ival  = ((uint32)tbuf[0]) << 18;
		ival |= ((uint32)tbuf[1]) << 12;
		ival |= ((uint32)tbuf[2]) <<  6;
		ival |= ((uint32)tbuf[3]);
		optr[2] = (octet) (ival & 0xff); ival >>= 8;
		optr[1] = (octet) (ival & 0xff); ival >>= 8;
		optr[0] = (octet) (ival & 0xff);
		optr += 3;
		break;
	    case 3:
		if (eq) {
		    ival  = ((uint32)tbuf[0]) << 10;
		    ival |= ((uint32)tbuf[1]) <<  4;
		    ival |= ((uint32)tbuf[2]) >>  2;
		    optr[1] = (octet) (ival & 0xff); ival >>= 8;
		    optr[0] = (octet) (ival & 0xff);
		    optr += 2;
		} else {
		    /* Not enough remaining chars.  return with leftovers */
		    iptr = mark;
		}
		if (inRead) *inRead = (int) (iptr - inBuf);
		return (int) (optr - outBuf);
	    case 2:
		if (eq) {
		    ival  = ((uint32)tbuf[0]) << 2;
		    ival |= ((uint32)tbuf[1]) >> 4;
		    optr[0] = (octet) (ival & 0xff);
		    optr += 1;
		} else {
		    /* Not enough remaining chars.  return with leftovers */
		    iptr = mark;
		}
		if (inRead) *inRead = (int) (iptr - inBuf);
		return (int) (optr - outBuf);
	    case 1:
		/*
		 * Should _NEVER_ see one char and 3 '='.
		 * If we do, just discard it, it's WRONG.
		 */
		if (!eq) {
		    /* Not enough remaining chars.  return with leftovers */
		    iptr = mark;
		}
		if (inRead) *inRead = (int) (iptr - inBuf);
		return (int) (optr - outBuf);
	    case 0:
		if (inRead) *inRead = (int) (iptr - inBuf);
		return (int) (optr - outBuf);
	}
    }
    /* NOTREACHED */
}

/**
 * Generate a 'sticky session' tag.
 * <P>
 * Converts an IP number and port into an Enhydra Director session affinity tag.
 * The port number and IP number must both be in network byte order
 * when passed to this function.  If you have a sockaddr_in structure,
 * pass sin.sin_port and sin.sin_addr.s_addr, and they will be in the
 * correct byte order.
 * <P>
 * An output buffer exactly nine bytes in length must be passed to 
 * receive the tag.  Eight bytes are for the tag itself, and the 
 * ninth receives the terminating null character.
 * 
 * @param ipAddr [in]  Unsigned 32 bit integer IP number in network byte order.
 * @param port   [in]  Unsigned 16 bit integer port in network byte order.
 * @param outBuf [out] Character array with at least 9 bytes allocated.
 */
extern void
edir_cookie_sticky_session_tag(uint32 ipAddr, uint16 port, char *outBuf)
{
    octet tbuf[6];
    memcpy((char *)(&tbuf[0]), (char *)(&ipAddr), 4);
    memcpy((char *)(&tbuf[4]), (char *)(&port), 2);
    edir_cookie_b64_encode(tbuf, 6, outBuf);
}

#ifdef DEBUG_MAIN
/**
 * Optional program entry point for testing and debugging.
 * <P>
 * This function is normally not compiled.
 * 
 * @param argc [in]  Number of command line arguments.
 * @param argv [in]  Command line arguments.
 * @return Integer exit status.
 */
int
main(int argc, char *argv[])
{
    octet	obuf[512], obuf2[512];
    char	cbuf[512], cbuf2[512];
    int		i, j, n, r, olen;
    for (i=1; i<argc; i++) {
	printf("s           = '%s'\n", argv[i]); fflush(stdout);

	n = edir_cookie_encode(argv[i], strlen(argv[i]), cbuf);
	printf("enc(s)      = '%s' (%d in, %d out)\n",cbuf,strlen(argv[i]), n);
	fflush(stdout);

	n = edir_cookie_decode(argv[i], obuf, &r);
	olen = n;
	printf("dec(s)      = '");
	for (j=0; j<n; j++) {
	    if ((obuf[j] >= ' ') && (obuf[j] <= '~')) putchar(obuf[j]);
	    else putchar('.');
	    fflush(stdout);
	}
	printf("' (%d in, %d out)\n", r, n); fflush(stdout);

	n = edir_cookie_decode(cbuf, obuf2, &r);
	printf("dec(enc(s)) = '"); fflush(stdout);
	for (j=0; j<n; j++) {
	    if ((obuf2[j] >= ' ') && (obuf2[j] <= '~')) putchar(obuf2[j]);
	    else putchar('.');
	    fflush(stdout);
	}
	printf("' (%d in, %d out)\n", r, n); fflush(stdout);

	n = edir_cookie_encode(obuf, olen, cbuf2);
	printf("enc(dec(s)) = '%s' (%d in, %d out)\n", cbuf2, olen, n);
	printf("\n\n"); fflush(stdout);
    }
}
#endif

#ifdef GEN_DECODE_TABLE
/**
 * Optional program entry point to automatically generate decodings table.
 * <P>
 * This entry point can be conditionally be compiled to allow the
 * encodings table to be reliably generated from the encodings table.
 * This is provided since writing the decodings table is an error prone task.
 * <P>
 * To use this, compile this module with -DGEN_DECODE_TABLE, run
 * the resulting executable and then replace the 'decodings' table
 * with the output.
 * 
 * @param argc [in]  Number of command line arguments.
 * @param argv [in]  Command line arguments.
 * @return Integer exit status.
 */
main()
{
    char newtable[256];
    char *comma;
    int i, j;
    for (i=0; i<256; i++) newtable[i] = -1;
    for (i=0; i<64; i++) newtable[encodings[i]] = i;
    printf("static char decodings[256] = {\n");
    for (i=0; i<16; i++) {
	comma = "";
	printf("    /*%3d*/  ", i * 16);
	for (j=0; j<16; j++) {
	    printf("%s%2d", comma, (int) (newtable[(i * 16) + j]));
	    comma = ", ";
	}
	if (i < 15)
	    printf(",\n");
	else
	    printf("\n");
    }
    printf("};\n");
    exit(0);
}
#endif

