/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_ipc.h,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */

#ifndef __EDIR_IPC_H__
#define __EDIR_IPC_H__ 1
#ifdef __cplusplus
extern "C" {
#endif

#define EDIR_IPC_HANDLE_TYPE_CONFIG	0
#define EDIR_IPC_HANDLE_TYPE_MUTEX	1
#define EDIR_IPC_HANDLE_TYPE_SHMEM	2
#define EDIR_IPC_HANDLE_TYPE_REFCOUNT	3
#define EDIR_IPC_HANDLE_TYPE_ISVALID(T) (((T) >= 0) && ((T) <= 3))

#define EDIR_IPC_OBJECT_TYPE_NONE		0
#define EDIR_IPC_OBJECT_TYPE_STRING	1
#define EDIR_IPC_OBJECT_TYPE_INTEGER	2
#define EDIR_IPC_OBJECT_TYPE_FILEDESC	3
#define EDIR_IPC_OBJECT_TYPE_SCOREBOARD	4
#define EDIR_IPC_OBJECT_TYPE_ISVALID(T) (((T) >= 0) && ((T) <= 4))

typedef struct edir_ipc_handle {
    int			  handle_type;	/* type of handle. */
    int			  object_type;	/* Type of underlying object */
    int			  nrefs;	/* Ref count. */
    void		  *pData;	/* Private data */
    struct edir_ipc_handle  *share;	/* Whose object we share */
} edir_ipc_handle_t;

#define EDIR_IPC_DESTROY_SHARE(H, E) \
    { \
	if ((H) && (H)->share) { \
	    switch((H)->share->handle_type) { \
	       case EDIR_IPC_HANDLE_TYPE_MUTEX: \
		    edir_mutex_destroy((void *)((H)->share), (E)); \
		    break; \
	       case EDIR_IPC_HANDLE_TYPE_SHMEM: \
		    edir_shmem_destroy((void *)((H)->share), (E)); \
		    break; \
	       case EDIR_IPC_HANDLE_TYPE_REFCOUNT: \
		    edir_refcount_destroy((void *)((H)->share), (E)); \
		    break; \
	    } \
	    (H)->share = NULL; \
	    if (*(E)) EDIR_THROW(*(E)); \
	} \
    }

#define EDIR_IPC_RELEASE_SHARE(H, E) \
    { \
	if ((H) && (H)->share) { \
	    switch((H)->share->handle_type) { \
	       case EDIR_IPC_HANDLE_TYPE_MUTEX: \
		    edir_mutex_release((void *)((H)->share), (E)); \
		    break; \
	       case EDIR_IPC_HANDLE_TYPE_SHMEM: \
		    edir_shmem_release((void *)((H)->share), (E)); \
		    break; \
	       case EDIR_IPC_HANDLE_TYPE_REFCOUNT: \
		    edir_refcount_release((void *)((H)->share), (E)); \
		    break; \
	    } \
	    (H)->share = NULL; \
	    if (*(E)) EDIR_THROW(*(E)); \
	} \
    }

/**
 * Return a pointer to the underlying handle object.
 * <P>
 * For CONFIG handles, returns a reference to the value string.
 * <P>
 * The string pointed to MUST NOT be modified!
 * 
 * @param handle [in] The handle from which to obtain the pointer.
 * @return Reference to the underlying object, or NULL if not valid.
 */
extern void *
edir_ipc_config_handle_get_object(edir_ipc_handle_t *handle);

/**
 * Return a string representation of the underlying object's name.  
 * <P>
 * For CONFIG handles, returns a copy of the value string allocated
 * using edir_malloc().
 * <P>
 * This string must be freed using edir_free().
 * 
 * @param handle [in] The handle from which to obtain the name.
 * @return The name of the underlying object, or NULL if not valid.
 */
extern char *
edir_ipc_config_handle_get_object_name(edir_ipc_handle_t *handle);

/**
 * Convert a string into an IPC handle of type EDIR_IPC_HANDLE_TYPE_CONFIG.
 * <P>
 * This allows explicit configuration values to be used as well as
 * values parsed from a config source/file.
 * 
 * @param str [in] The string to wrap in a handle object.
 * @return A new handle object with a copy of the string.
 */
extern edir_ipc_handle_t * edir_ipc_make_config_handle(char *str);

/**
 * Convert a string into an IPC handle of type EDIR_IPC_HANDLE_TYPE_CONFIG.
 * <P>
 * This allows explicit configuration values to be used as well as
 * values parsed from a config source/file.
 * 
 * @param str [in] The string to wrap in a handle object.
 * @return A new handle object with a copy of the string.
 */
extern void edir_ipc_destroy_config_handle(edir_ipc_handle_t *handle);

#ifdef __cplusplus
} /* Extern "C" */
#endif
#endif /* __EDIR_IPC_H__ */

