/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_kvtable.h,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */
#ifndef __EDIR_KVTABLE_H__
#define __EDIR_KVTABLE_H__ 1
#include "edir_table.h"
#ifdef __cplusplus
extern "C" {
#endif

/*
 * Maximum length of a key.  Needed to avoid the penalty of malloc()
 * on every single lookup when converting keys to the desired case.
 */
#define EDIR_KVTABLE_MAX_KEY_LEN	256


#define EDIR_KVTABLE_TO_UPPER		0x01
#define EDIR_KVTABLE_TO_LOWER		0x02
#define EDIR_KVTABLE_PRESERVE		0x04
#define EDIR_KVTABLE_TO_UPPER_PRESERVE	0x05
#define EDIR_KVTABLE_TO_LOWER_PRESERVE	0x06
#define EDIR_KVTABLE_DEFAULT		0x00

/*
 * Internal structure that keeps track of a kvtable.
 * <P>
 * Do NOT mess with the fields in this structure unless you like core files.
 * The kvtable routines should work fine in a multithreaded environment
 * as long as only one thread at a time has access to any individual
 * kvtable structure.  No global structures are used.
 */
typedef struct edir_kvtable {
    edir_HashTable	table;
    edir_HashSearch	search;
    int32		options;
    char		keybuf[EDIR_KVTABLE_MAX_KEY_LEN + 1];
    char		*key;
    int32		arrayLen;
    char		**array;
} edir_kvtable_t;

extern edir_kvtable_t *
edir_kvtable_create(int32 keyCase);

extern void
edir_kvtable_destroy(edir_kvtable_t *table);

extern void
edir_kvtable_add(edir_kvtable_t *table, char *key, char *value);

extern void
edir_kvtable_set(edir_kvtable_t *table, char *key, char *value);

extern char *
edir_kvtable_find(edir_kvtable_t *table, char *key);

extern char **
edir_kvtable_find_all(edir_kvtable_t *table, char *key, int32 *nvals);

extern char **
edir_kvtable_first(edir_kvtable_t *table, char **keyp, int32 *nvals);

extern char **
edir_kvtable_next(edir_kvtable_t *table, char **keyp, int32 *nvals);

extern void
edir_kvtable_remove(edir_kvtable_t *table, char *key);

#ifdef __cplusplus
} /* extern "C" */
#endif
#endif /* __EDIR_KVTABLE_H__ */

