/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_load_daemon_start.c,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */

#include "edir.h"
#include "edir_conf_file.h"

/**
 * The user ID of the daemon process, if we started as root.
 */
static uid_t	daemonUID = -1;
static int	daemonUIDSet = 0;

/**
 * The group ID of the daemon process, if we started as root.
 */
static gid_t	daemonGID = -1;
static int	daemonGIDSet = 0;

extern int
edir_load_daemon_start(char *progName, char *configFile, edir_exception_t **exc)
{
    static char		funcName = "edir_load_start_daemon";
    int			daemonPid = 0;
    int			rfd=-1, wfd=-1;

    daemonPid = edir_daemon_fork(progName, &rfd, &wfd, exc);
    If (*exc) EDIR_THROW(*exc);
    /* daemonPid is only negative if an exception is thrown */
    if (daemonPid == 0) {
	/* In daemon process.  rfd is always zero and wfd is always 1,2 */
	/*
	 * First, see if we need to become a 'nobody'
	 */
	if (daemonGIDSet) {
	    if ((geteuid() == 0) || (getuid() != geteuid()))
		if (setgid(daemonGID) < 0) {
		    /* Be very anal retentive in root or setuid env! */
		    exit(1);
		}
		/* Even more anal retention in root case */
		if (getgid() != daemonGID) exit(2);
	    } else {
		/* If just normal non-root non-setuid user, go ahead and try
		 * The user can only run as himself anyway.
		 */
		setgid(daemonGID);
	    }
	}
	if (daemonUIDSet) {
	    if ((geteuid() == 0) || (getuid() != geteuid()))
		if (setuid(daemonUID) < 0) {
		    /* Be very anal retentive in root or setuid env! */
		    exit(1);
		}
		/* Even more anal retention in root case */
		if (getuid() != daemonUID) exit(2);
	    } else {
		/* If just normal non-root non-setuid user, go ahead and try
		 * The user can only run as himself anyway.
		 */
		setuid(daemonUID);
	    }
	}

	/*
	 * Now start up the child program.
	 */
	edir_load_daemon_child_exec(progName, configFile);
	exit(0);
    }

    /*
     * Still in (grand)parent process at and after this point. 
     */

    /*
     * wait for the daemon to tell us the scoreboard is ready.
     * daemon will send a zero byte over the pipe.
     */
    while (((n = read(rfd, &inb, 1)) < 0) && (errno == EINTR)) ;
    if (n < 0) {
	*exc = edir_exception_create(EDIR_EXCEPT_DAEMON,
		"Can't receive daemon status: @errstr@ (errnum=@errnum@)");
	EDIR_THROW(*exc);
    }
    if (n != 1) {
	*exc = edir_exception_create(EDIR_EXCEPT_DAEMON,
		"Didn't receive expected single byte from daemon");
	EDIR_THROW(*exc);
    }
    if (inb != 0) {
	*exc = edir_exception_create(EDIR_EXCEPT_DAEMON,
		"Got failure status 0x%02x from daemon", ((int)0)+inb);
	EDIR_THROW(*exc);
    }

    /*
     * Daemon has said "I'm OK, how are you?"
     * Now we open the scoreboard.  If successful, we send
     * back "I'm OK too, carry on!"  Daemon will then go into
     * its normal loop and we'll have our scoreboard ref.
     */


    return;
    
except:
    if (wfd >= 0) {
	void (*savesig)(int);
	inb = 0xff; /* "I'm NOT OK" */
	savesig = signal(SIGPIPE, SIG_IGN); /* Don't want SIGPIPE */
	while ((write(wfd, &inb, 1) < 0) && (errno == EINTR)) ;
	close(wfd);
	signal(SIGPIPE, savesig); /* Restore previous disposition */
    }
    if (rfd >= 0) {
	close(rfd);
    }
    /* Try to make sure daemon is killed. */
    if (daemonPid > 1) {
	kill(daemonPid, 15);
	sleep(3);
	kill(daemonPid, 9);
    }
    return;
}

