/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_load_multiproc.h,v 1.1 2003/04/14 12:25:56 sinisa Exp $
 */

#ifndef __EDIR_LOAD_MULTIPROC_H__
#define __EDIR_LOAD_MULTIPROC_H__ 1
#ifdef __cplusplus
extern "C" {
#endif

#define EDIR_LOAD_DEFAULT_DAEMON_PATH	"/bin:/usr/bin"
#define EDIR_LOAD_DEFAULT_DAEMON_NAME	"edir_daemon"
#define EDIR_LOAD_DAEMON_MAX_TRIES	(5)

typedef struct edir_load_handles {
    char			*configFile;
    char			*daemonFile;
    char			*lockFile;
    char			*dataFile;
    edir_ipc_handle_t		*mutex;
    edir_ipc_handle_t		*shmem;
    edir_ipc_handle_t		*refcnt;	
    edir_load_sb_offsets_t	*scoreboard;
    edir_load_sb_app_t		*apps;
    edir_load_sb_server_t	*servers;
    int				destroyOnShutdown;
} edir_load_handles_t;

extern edir_load_handles_t *
edir_load_init(char *configFile, char *daemonFile, char *lockFile,
	     char *dataFile, edir_exception_t **exc);

extern void
edir_load_shutdown(edir_load_handles_t *handles, edir_exception_t **exc);

extern edir_load_handles_t *
edir_load_check(edir_load_handles_t *handles, edir_exception_t **exc);

/**
 * Allow a module to set up the "default" daemon program location.
 * 
 * @param prog [in] The name of the program to run as the daemon.
 */
extern void edir_load_daemon_set_default_path(char *prog);

/**
 * Set the UID of the watchdog daemon.  This is useful for
 * Apache, which initializes as root on most systems.
 * 
 * @param uid [in] The user ID to change to on init.
 */
extern void
edir_load_daemon_set_uid(uid_t uid);

/**
 * Set the GID of the watchdog daemon.  This is useful for
 * Apache, which initializes as root on most systems.
 * 
 * @param gid [in] The group ID to change to on init.
 */
extern void
edir_load_daemon_set_gid(gid_t gid);


#ifdef __cplusplus
} /* Extern "C" */
#endif
#endif /* __EDIR_LOAD_MULTIPROC_H__ */

