/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_log_line.c,v 1.1 2003/04/14 12:25:59 sinisa Exp $
 */

#include "edir.h"

int edir_log_is_daemon = 0;

#ifndef LOG_NOTICE
#define LOG_NOTICE 0
#endif

#ifndef LOG_ERR
#define LOG_ERR 1
#endif

#ifndef LOG_DEBUG
#define LOG_DEBUG 2
#endif

/**
 * Log a single line of text to the correct log channel.
 * <P>
 * This function should only be called by the general logging functions
 * in 'edir_log.c'.  The common line logging function has been split out
 * into a separate module to allow specific client implementations to
 * integrate logging into the native web server's mechanism.
 * 
 * @param level [in]  The logging level, as per syslog.
 * @param line  [in]  Single line of pre-formatted log text.
 */
extern void
edir_log_line(int level, char *line)
{
#ifdef HAVE_SYSLOG
    if (edir_log_is_daemon)
	syslog(level, "%s", line);
    else
#endif
    switch (level) {
	case LOG_ERR:
	    fprintf(stderr, "%s\n", line);
	    fflush(stderr);
	    break;
	case LOG_DEBUG:
	    printf("%s\n", line);
	    fflush(stdout);
	    break;
	case LOG_NOTICE:
	default:
	    printf("%s\n", line);
	    break;
    }
}

/**
 * Set whether the log system is in 'daemon' mode.
 * <P>
 * In the standard log module, this means "use syslog instead of printf".
 * It may or may not be used by other modules, such as Apache.
 * 
 * @param progName [in]  The name of the program doing the logging.
 * @param isDaemon [in]  True for daemon mode, false for "normal".
 */
extern void
edir_log_setdaemon(char *progName, int isDaemon)
{
    if (progName == NULL) progName = "Enhydra Director";
    if (isDaemon) {
	edir_log_is_daemon = 1;
#if defined(HAVE_OPENLOG) && defined(LOG_PID) && defined(LOG_DAEMON)
	closelog();
	openlog(progName, LOG_PID, LOG_DAEMON);
#endif
    } else {
	edir_log_is_daemon = 0;
    }
}

