/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_proto.h,v 1.1 2003/04/14 12:25:59 sinisa Exp $
 */

#ifndef __EDIR_PROTO_H__
#define __EDIR_PROTO_H__ 1
#ifdef __cplusplus
extern "C" {
#endif

/*
 * Enhydra Director Protocol Version 1.0.
 */
#define EDIR_PROTO_MAJOR_VERSION			1
#define EDIR_PROTO_MINOR_VERSION			0

#define EDIR_PKT_TYPE_AUTH			0
#define EDIR_PKT_SUBTYPE_AUTH_SVR			0
#define EDIR_PKT_SUBTYPE_AUTH_CLI			1
#define EDIR_PKT_SUBTYPE_AUTH_OK			2
#define EDIR_PKT_SUBTYPE_AUTH_FAIL		3

#define EDIR_PKT_TYPE_REQ				1
#define EDIR_PKT_SUBTYPE_REQ_DATA			0
#define EDIR_PKT_SUBTYPE_REQ_LASTDATA		1

#define EDIR_PKT_TYPE_FUNC			2
#define EDIR_PKT_SUBTYPE_FUNC_FAIL		0
#define EDIR_PKT_SUBTYPE_FUNC_OK			1
#define EDIR_PKT_SUBTYPE_FUNC_CALL		2
#define EDIR_PKT_SUBTYPE_FUNC_ONEWAY		3
#define EDIR_PKT_SUBTYPE_FUNC_END			4

#define EDIR_PKT_TYPE_RESP			3
#define EDIR_PKT_SUBTYPE_RESP_DATA		0
#define EDIR_PKT_SUBTYPE_RESP_LOGS		1
#define EDIR_PKT_SUBTYPE_RESP_WARNING		2
#define EDIR_PKT_SUBTYPE_RESP_ERROR		3
#define EDIR_PKT_SUBTYPE_RESP_END			4

#define EDIR_PKT_TYPE_HEADER			4
#define EDIR_PKT_SUBTYPE_HEADER_HEADER		0
#define EDIR_PKT_SUBTYPE_HEADER_HTTP		1
#define EDIR_PKT_SUBTYPE_HEADER_COOKIE		2
#define EDIR_PKT_SUBTYPE_HEADER_LAST		15

#define EDIR_PKT_TYPE_CTRL_CLI			5
#define EDIR_PKT_SUBTYPE_CTRL_CLI_ACTIVATE	0
#define EDIR_PKT_SUBTYPE_CTRL_CLI_DEACTIVATE	1
#define EDIR_PKT_SUBTYPE_CTRL_CLI_CLOSE		2

#define EDIR_PKT_TYPE_CTRL_SVR			6
#define EDIR_PKT_SUBTYPE_CTRL_SVR_REFRESH		0

#define EDIR_PKT_TYPE_PROTO			15
#define EDIR_PKT_SUBTYPE_PROTO_FATAL		1
#define EDIR_PKT_SUBTYPE_PROTO_VERSION		15

#define EDIR_PKT_OPENFLAGS_OPENCB			0x01
#define EDIR_PKT_OPENFLAGS_HEADERCB		0x02

#define EDIR_PROTO_HEADER_LEN              	4

#define EDIR_PROTO_MAX_CHALLENGE_LEN		100
#define EDIR_PROTO_MAX_AUTH_REPLY_LEN		100
#define EDIR_PROTO_MAX_CALLBACK_LEN		10000
#define EDIR_PROTO_MAX_FUNC_RESULT_LEN		10000

#define EDIR_PROTO_BUFSIZE			10000
#define EDIR_PROTO_MD5_HASH_SIZE			16

/**
 * In-memory copy of a packet's header, in machine byte order.
 * 
 * @field type		The packet type as per the spec.
 * @field subtype	The packet type as per the spec.
 * @field length	The length of the packet's data area.
 */ 
typedef struct {
    int16		type;
    int16		subtype;
    int32		length;	
} edir_header_t;

extern void
edir_proto_make_header(octet *pkt, int16 type, int16 subtype, int32 length);

extern void
edir_proto_make_header_header(octet *pkt, int16 subtype,
			    int32 nameLen, int32 valueLen);

extern void
edir_proto_send_packet(edir_connection_t *conn, int type, int subtype,
		     edir_exception_t **exc);

extern void
edir_proto_send_version(edir_connection_t *conn, int major, int minor,
		     edir_exception_t **exc);

extern void
edir_proto_send_header_packet(edir_connection_t *conn, edir_exception_t **exc );

extern void
edir_proto_finish_send(edir_connection_t *conn, edir_exception_t **exc);

extern void
edir_proto_receive_packet_header(edir_connection_t *conn, edir_exception_t **exc);

extern void
edir_proto_receive_packet(edir_connection_t *conn, edir_exception_t **exc);

extern void
edir_proto_receive_packet_data(edir_connection_t *conn, edir_exception_t **exc);

extern void
edir_proto_create_packetdatabuf(edir_connection_t *conn);

extern void
edir_proto_destroy_packetdatabuf(edir_connection_t *conn);

#ifdef __cplusplus
} /* Extern "C" */
#endif
#endif /* __EDIR_PROTO_H__ */

