/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: edir_xml.c,v 1.1 2003/04/14 12:25:59 sinisa Exp $
 */

#include "parser.h"
#include "edir.h"
#include "edir_xml.h"

/*
 * Setting this enables DTD validation checking.
 */
extern int xmlDoValidityCheckingDefaultValue;

/*
 * Number of embedded DTDs
 */
extern int  edir_dtd_count;

/*
 * Names of embedded DTDs
 */
extern char *edir_dtd_names[];

/*
 * Embeded DTD file contents.
 */
extern char *edir_dtd_bufs[];

/*
 * libXml function to generate an input stream from a buffer.
 */
extern xmlParserInputPtr xmlNewStringInputStream(xmlParserCtxtPtr,
						    const xmlChar *);

/*
 * Embeded DTD init function.
 */
extern void edir_dtd_init();

/**
 * Default pass-through filter replacement for <code>resolveEntity</code>.
 * 
 * @param ctx      [in,out] Opaque filter data object.
 * @param publicId [in]	    Public ID string.
 * @param systemId [in]     System ID string.
 * @return Input stream pointer for the newly resolved entity.
 */
static xmlParserInputPtr 
edir_xml_sax_resolveEntity(void *ctx, const xmlChar *publicId,
			   const xmlChar *systemId)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    return (filt->oldsax->resolveEntity)(filt->ctx, publicId, systemId);
}

/**
 * Default pass-through filter replacement for <code>internalSubset</code>.
 * 
 * @param ctx        [in,out] Opaque filter data object.
 * @param name       [in]     Token name.
 * @param ExternalID [in]     External id.
 * @param SystemID   [in]     System id.
 */
static void
edir_xml_sax_internalSubset(void *ctx, const xmlChar *name,
		            const xmlChar *ExternalID, const xmlChar *SystemID)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->internalSubset)(filt->ctx, name, ExternalID, SystemID);
}

/**
 * Default pass-through filter replacement for <code>internalSubset</code>.
 * 
 * @param ctx        [in,out] Opaque filter data object.
 * @param name       [in]     Token name.
 * @param ExternalID [in]     External id.
 * @param SystemID   [in]     System id.
 */
static void
edir_xml_sax_externalSubset(void *ctx, const xmlChar *name,
		            const xmlChar *ExternalID, const xmlChar *SystemID)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->externalSubset)(filt->ctx, name, ExternalID, SystemID);
}

/**
 * Default pass-through filter replacement for <code>getEntity</code>.
 * 
 * @param ctx        [in,out] Opaque filter data object.
 * @param name       [in]     Token name.
 * @return Pointer to an entity structure.
 */
static xmlEntityPtr
edir_xml_sax_getEntity(void *ctx, const xmlChar *name)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    return (filt->oldsax->getEntity)(filt->ctx, name);
}

/**
 * Default pass-through filter replacement for <code>getParameterEntity</code>.
 * 
 * @param ctx        [in,out] Opaque filter data object.
 * @param name       [in]     Token name.
 * @return Pointer to an entity structure.
 */
static xmlEntityPtr
edir_xml_sax_getParameterEntity(void *ctx, const xmlChar *name)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    return (filt->oldsax->getParameterEntity)(filt->ctx, name);
}

/**
 * Default pass-through filter replacement for <code>entityDecl</code>.
 * 
 * @param ctx      [in,out] Opaque filter data object.
 * @param name     [in]     Token name.
 * @param type     [in]     Entity type.
 * @param publicId [in]	    Public ID string.
 * @param systemId [in]     System ID string.
 * @param content  [in]     Entity content.
 */
static void
edir_xml_sax_entityDecl(void *ctx, const xmlChar *name, int type,
		        const xmlChar *publicId, const xmlChar *systemId,
		        xmlChar *content)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->entityDecl)(filt->ctx, name, type, publicId,
				   systemId, content);
}

/**
 * Default pass-through filter replacement for <code>notationDecl</code>.
 * 
 * @param ctx      [in,out] Opaque filter data object.
 * @param name     [in]     Token name.
 * @param publicId [in]	    Public ID string.
 * @param systemId [in]     System ID string.
 */
static void
edir_xml_sax_notationDecl(void *ctx, const xmlChar *name,
		          const xmlChar *publicId, const xmlChar *systemId)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->notationDecl)(filt->ctx, name, publicId, systemId);
}

/**
 * Default pass-through filter replacement for <code>attributeDecl</code>.
 * 
 * @param ctx          [in,out] Opaque filter data object.
 * @param elem         [in]     Element name.
 * @param name         [in]     Token name.
 * @param type         [in]     Attribute type.
 * @param def          [in]     Attribute def.
 * @param defaultValue [in]     Default attribute value.
 * @param tree         [in]     tree pointer.
 */
static void
edir_xml_sax_attributeDecl(void *ctx, const xmlChar *elem,
		           const xmlChar *name, int type, int def,
		           const xmlChar *defaultValue, xmlEnumerationPtr tree)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->attributeDecl)(filt->ctx, elem, name, type, def,
				      defaultValue, tree);
}

/**
 * Default pass-through filter replacement for <code>elementDecl</code>.
 * 
 * @param ctx          [in,out] Opaque filter data object.
 * @param elem         [in]     Element name.
 * @param name         [in]     Token name.
 * @param type         [in]     Attribute type.
 * @param def          [in]     Attribute def.
 * @param defaultValue [in]     Default attribute value.
 * @param tree         [in]     tree pointer.
 */
static void
edir_xml_sax_elementDecl(void *ctx, const xmlChar *name,
		         int type, xmlElementContentPtr content)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->elementDecl)(filt->ctx, name, type, content);
}

/**
 * Default pass-through filter replacement for <code>unparsedEntityDecl</code>.
 * 
 * @param ctx          [in,out] Opaque filter data object.
 * @param name         [in]     Token name.
 * @param publicId     [in]	Public ID string.
 * @param systemId     [in]     System ID string.
 * @param notationName [in]     Notation name.
 */
static void
edir_xml_sax_unparsedEntityDecl(void *ctx,
                          const xmlChar *name, const xmlChar *publicId,
			  const xmlChar *systemId, const xmlChar *notationName)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->unparsedEntityDecl)(filt->ctx, name, publicId,
					   systemId, notationName);
}

/**
 * Default pass-through filter replacement for <code>setDocumentLocator</code>.
 * 
 * @param ctx  [in,out] Opaque filter data object.
 * @param loc  [in]     Document locator structure.
 */
static void
edir_xml_sax_setDocumentLocator(void *ctx, xmlSAXLocatorPtr loc)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->setDocumentLocator)(filt->ctx, loc);
}

/**
 * Default pass-through filter replacement for <code>setDocument</code>.
 * 
 * @param ctx  [in,out] Opaque filter data object.
 */
static void
edir_xml_sax_startDocument(void *ctx)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->startDocument)(filt->ctx);
}

/**
 * Default pass-through filter replacement for <code>endDocument</code>.
 * 
 * @param ctx  [in,out] Opaque filter data object.
 */
static void
edir_xml_sax_endDocument(void *ctx)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->endDocument)(filt->ctx);
}

/**
 * Default pass-through filter replacement for <code>startElement</code>.
 * 
 * @param ctx  [in,out] Opaque filter data object.
 * @param name [in]     Element name.
 * @param atts [in]     List of attributes.
 */
static void
edir_xml_sax_startElement(void *ctx, const xmlChar *name, const xmlChar **atts)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->startElement)(filt->ctx, name, atts);
}

/**
 * Default pass-through filter replacement for <code>endElement</code>.
 * 
 * @param ctx  [in,out] Opaque filter data object.
 * @param name [in]     Element name.
 */
static void
edir_xml_sax_endElement(void *ctx, const xmlChar *name)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->endElement)(filt->ctx, name);
}

/**
 * Default pass-through filter replacement for <code>reference</code>.
 * 
 * @param ctx  [in,out] Opaque filter data object.
 * @param name [in]     Reference name.
 */
static void
edir_xml_sax_reference(void *ctx, const xmlChar *name)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->reference)(filt->ctx, name);
}

/**
 * Default pass-through filter replacement for <code>characters</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @param ch  [in]     String containing the characters.
 * @param len [in]     Number of characters.
 */
static void
edir_xml_sax_characters(void *ctx, const xmlChar *ch, int len)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->characters)(filt->ctx, ch, len);
}

/**
 * Default pass-through filter replacement for
 * <code>ignorableWhitespace</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @param ch  [in]     String containing the characters.
 * @param len [in]     Number of characters.
 */
static void
edir_xml_sax_ignorableWhitespace(void *ctx, const xmlChar *ch, int len)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->ignorableWhitespace)(filt->ctx, ch, len);
}

/**
 * Default pass-through filter replacement for
 * <code>processingInstruction</code>.
 * 
 * @param ctx    [in,out] Opaque filter data object.
 * @param target [in]     Target name.
 * @param data   [in]     Data characters.
 */
static void
edir_xml_sax_processingInstruction(void *ctx, const xmlChar *target,
			           const xmlChar *data)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->processingInstruction)(filt->ctx, target, data);
}

/**
 * Default pass-through filter replacement for <code>comment</code>.
 * 
 * @param ctx   [in,out] Opaque filter data object.
 * @param value [in]     The comment string.
 */
static void
edir_xml_sax_comment(void *ctx, const xmlChar *value)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->comment)(filt->ctx, value);
}

/**
 * Default pass-through filter replacement for <code>cdataBlock</code>.
 * 
 * @param ctx   [in,out] Opaque filter data object.
 * @param value [in]     The character data
 * @param len   [in]     Number of bytes of character data.
 */
static void
edir_xml_sax_cdataBlock(void *ctx, const xmlChar *value, int len)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    (filt->oldsax->cdataBlock)(filt->ctx, value, len);
}

/**
 * Default pass-through filter replacement for <code>warning</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @param msg [in]     Printf-style format string.
 * @param ... [in]     Printf-style variadic argument list.
 */
static void
edir_xml_sax_warning(void *ctx, const char *msg, ...)
{
    va_list ap;
    char buf[512];
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    va_start(ap, msg);
    edir_vsnprintf(buf, 511, msg, ap);
    va_end(ap);
    buf[510] = '\0';
    (filt->oldsax->warning)(filt->ctx, "%s", buf);
}

/**
 * Default pass-through filter replacement for <code>error</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @param msg [in]     Printf-style format string.
 * @param ... [in]     Printf-style variadic argument list.
 */
static void
edir_xml_sax_error(void *ctx, const char *msg, ...)
{
    va_list ap;
    char buf[512];
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    va_start(ap, msg);
    edir_vsnprintf(buf, 511, msg, ap);
    va_end(ap);
    buf[510] = '\0';
    (filt->oldsax->error)(filt->ctx, "%s", buf);
}

/**
 * Default pass-through filter replacement for <code>fatalError</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @param msg [in]     Printf-style format string.
 * @param ... [in]     Printf-style variadic argument list.
 */
static void
edir_xml_sax_fatalError(void *ctx, const char *msg, ...)
{
    va_list ap;
    char buf[512];
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    va_start(ap, msg);
    edir_vsnprintf(buf, 511, msg, ap);
    va_end(ap);
    buf[510] = '\0';
    (filt->oldsax->fatalError)(filt->ctx, "%s", buf);
}

/**
 * Default pass-through filter replacement for <code>isStandalone</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @return True if standalone.
 */
static int
edir_xml_sax_isStandalone(void *ctx)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    return (filt->oldsax->isStandalone)(filt->ctx);
}

/**
 * Default pass-through filter replacement for <code>hasInternalSubset</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @return True if has internal subset.
 */
static int
edir_xml_sax_hasInternalSubset(void *ctx)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    return (filt->oldsax->hasInternalSubset)(filt->ctx);
}

/**
 * Default pass-through filter replacement for <code>hasExternalSubset</code>.
 * 
 * @param ctx [in,out] Opaque filter data object.
 * @return True if has external subset.
 */
static int
edir_xml_sax_hasExternalSubset(void *ctx)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    return (filt->oldsax->hasExternalSubset)(filt->ctx);
}

/**
 * Install a pass-through filter on a libxml parsing context.
 * <P>
 * This function replaces the SAX handler, the userData, and the
 * validation context error functions with pass-through versions that
 * allow specific handlers to be 'overridden' while preserving the
 * functionality of the existing handlers.  This allows the default
 * libxml DOM building handlers to be used while redirecting error
 * messages to a more useful place for Enhydra Director.
 * <P>
 * First, all handlers in the original SAX block are replaced with
 * pass-through equivalents if the original handler is not NULL.
 * Next, all handler pointers in the <code>filter</code> SAX block
 * are copied to the original SAX block.
 * Finally, the userData is replaced with a <code>edir_conf_xml_sax_filter_t
 * </code> pointer which contains filter bookkeeping information and the
 * new caller-specified clientData (as field 'data').  The overriding
 * handler functions are expected to be familiar with the 'data' field.
 * 
 * @param ctxt       [in,out] The parsing context to filter.
 * @param filter     [in,out] The overriding handlers for this filter.
 * @param clientData [in]     Optional caller-specified data that gets placed
 * 			      in the filter's 'data' field.
 */
extern void
edir_xml_sax_install_filter(xmlParserCtxtPtr ctxt, xmlSAXHandlerPtr filter,
			    void *clientData)
{
    edir_conf_xml_sax_filter_t	*filt = NULL;
    xmlSAXHandlerPtr		newsax = NULL;
    newsax = (xmlSAXHandlerPtr) edir_malloc(sizeof(xmlSAXHandler));
    memset((char *)newsax, 0, sizeof(xmlSAXHandler));
    filt = (edir_conf_xml_sax_filter_t *)
	    edir_malloc(sizeof(edir_conf_xml_sax_filter_t));
    memset((char *)filt, 0, sizeof(edir_conf_xml_sax_filter_t));
    filt->oldsax = ctxt->sax;
    filt->sax = newsax;
    filt->ctx = ctxt->userData;
    filt->ctxt = ctxt;
    filt->data = clientData;

    /*
     * Install default wrappers for any callbacks set in the original
     * SAX block.
     */
    if (filt->oldsax->internalSubset != NULL)
	filt->sax->internalSubset = edir_xml_sax_internalSubset;
    if (filt->oldsax->externalSubset != NULL)
	filt->sax->externalSubset = edir_xml_sax_externalSubset;
    if (filt->oldsax->isStandalone != NULL)
	filt->sax->isStandalone = edir_xml_sax_isStandalone;
    if (filt->oldsax->hasInternalSubset != NULL)
	filt->sax->hasInternalSubset = edir_xml_sax_hasInternalSubset;
    if (filt->oldsax->hasExternalSubset != NULL)
	filt->sax->hasExternalSubset = edir_xml_sax_hasExternalSubset;
    if (filt->oldsax->resolveEntity != NULL)
	filt->sax->resolveEntity = edir_xml_sax_resolveEntity;
    if (filt->oldsax->getEntity != NULL)
	filt->sax->getEntity = edir_xml_sax_getEntity;
    if (filt->oldsax->entityDecl != NULL)
	filt->sax->entityDecl = edir_xml_sax_entityDecl;
    if (filt->oldsax->notationDecl != NULL)
	filt->sax->notationDecl = edir_xml_sax_notationDecl;
    if (filt->oldsax->attributeDecl != NULL)
	filt->sax->attributeDecl = edir_xml_sax_attributeDecl;
    if (filt->oldsax->elementDecl != NULL)
	filt->sax->elementDecl = edir_xml_sax_elementDecl;
    if (filt->oldsax->unparsedEntityDecl != NULL)
	filt->sax->unparsedEntityDecl = edir_xml_sax_unparsedEntityDecl;
    if (filt->oldsax->setDocumentLocator != NULL)
	filt->sax->setDocumentLocator = edir_xml_sax_setDocumentLocator;
    if (filt->oldsax->startDocument != NULL)
	filt->sax->startDocument = edir_xml_sax_startDocument;
    if (filt->oldsax->endDocument != NULL)
	filt->sax->endDocument = edir_xml_sax_endDocument;
    if (filt->oldsax->startElement != NULL)
	filt->sax->startElement = edir_xml_sax_startElement;
    if (filt->oldsax->endElement != NULL)
	filt->sax->endElement = edir_xml_sax_endElement;
    if (filt->oldsax->reference != NULL)
	filt->sax->reference = edir_xml_sax_reference;
    if (filt->oldsax->characters != NULL)
	filt->sax->characters = edir_xml_sax_characters;
    if (filt->oldsax->ignorableWhitespace != NULL)
	filt->sax->ignorableWhitespace = edir_xml_sax_ignorableWhitespace;
    if (filt->oldsax->processingInstruction != NULL)
	filt->sax->processingInstruction = edir_xml_sax_processingInstruction;
    if (filt->oldsax->comment != NULL)
	filt->sax->comment = edir_xml_sax_comment;
    if (filt->oldsax->warning != NULL)
	filt->sax->warning = edir_xml_sax_warning;
    if (filt->oldsax->error != NULL)
	filt->sax->error = edir_xml_sax_error;
    if (filt->oldsax->fatalError != NULL)
	filt->sax->fatalError = edir_xml_sax_fatalError;
    if (filt->oldsax->getParameterEntity != NULL)
	filt->sax->getParameterEntity = edir_xml_sax_getParameterEntity;
    if (filt->oldsax->cdataBlock != NULL)
	filt->sax->cdataBlock = edir_xml_sax_cdataBlock;

    /*
     * Now install the overrides from the filter.
     */
    if (filter->internalSubset != NULL)
	filt->sax->internalSubset = filter->internalSubset;
    if (filter->externalSubset != NULL)
	filt->sax->externalSubset = filter->externalSubset;
    if (filter->isStandalone != NULL)
	filt->sax->isStandalone = filter->isStandalone;
    if (filter->hasInternalSubset != NULL)
	filt->sax->hasInternalSubset = filter->hasInternalSubset;
    if (filter->hasExternalSubset != NULL)
	filt->sax->hasExternalSubset = filter->hasExternalSubset;
    if (filter->resolveEntity != NULL)
	filt->sax->resolveEntity = filter->resolveEntity;
    if (filter->getEntity != NULL)
	filt->sax->getEntity = filter->getEntity;
    if (filter->entityDecl != NULL)
	filt->sax->entityDecl = filter->entityDecl;
    if (filter->notationDecl != NULL)
	filt->sax->notationDecl = filter->notationDecl;
    if (filter->attributeDecl != NULL)
	filt->sax->attributeDecl = filter->attributeDecl;
    if (filter->elementDecl != NULL)
	filt->sax->elementDecl = filter->elementDecl;
    if (filter->unparsedEntityDecl != NULL)
	filt->sax->unparsedEntityDecl = filter->unparsedEntityDecl;
    if (filter->setDocumentLocator != NULL)
	filt->sax->setDocumentLocator = filter->setDocumentLocator;
    if (filter->startDocument != NULL)
	filt->sax->startDocument = filter->startDocument;
    if (filter->endDocument != NULL)
	filt->sax->endDocument = filter->endDocument;
    if (filter->startElement != NULL)
	filt->sax->startElement = filter->startElement;
    if (filter->endElement != NULL)
	filt->sax->endElement = filter->endElement;
    if (filter->reference != NULL)
	filt->sax->reference = filter->reference;
    if (filter->characters != NULL)
	filt->sax->characters = filter->characters;
    if (filter->ignorableWhitespace != NULL)
	filt->sax->ignorableWhitespace = filter->ignorableWhitespace;
    if (filter->processingInstruction != NULL)
	filt->sax->processingInstruction = filter->processingInstruction;
    if (filter->comment != NULL)
	filt->sax->comment = filter->comment;
    if (filter->warning != NULL) {
	filt->sax->warning = filter->warning;
	filt->vctxt.error = ctxt->vctxt.error;
	ctxt->vctxt.error = filter->error;
	filt->vctxt.warning = ctxt->vctxt.warning;
	ctxt->vctxt.warning = filter->warning;
	filt->vctxt.userData = ctxt->vctxt.userData;
	ctxt->vctxt.userData = filt;
    }
    if (filter->error != NULL) {
	filt->sax->error = filter->error;
	filt->vctxt.error = ctxt->vctxt.error;
	ctxt->vctxt.error = filter->error;
	filt->vctxt.warning = ctxt->vctxt.warning;
	ctxt->vctxt.warning = filter->warning;
	filt->vctxt.userData = ctxt->vctxt.userData;
	ctxt->vctxt.userData = filt;
    }
    if (filter->fatalError != NULL)
	filt->sax->fatalError = filter->fatalError;
    if (filter->getParameterEntity != NULL)
	filt->sax->getParameterEntity = filter->getParameterEntity;
    if (filter->cdataBlock != NULL)
	filt->sax->cdataBlock = filter->cdataBlock;

    ctxt->userData = (void *) filt;
    ctxt->sax = filt->sax;
    return;
}

/**
 * Remove a pass-through filter from a libxml parsing context.
 * <P>
 * Restores the original SAX handler block to the context.
 * 
 * @param ctxt [in,out] The parsing context to remove the filter from.
 */
extern void
edir_xml_sax_remove_filter(xmlParserCtxtPtr ctxt)
{
    edir_conf_xml_sax_filter_t	*filt;
    filt = (edir_conf_xml_sax_filter_t *)(ctxt->userData);
    if (filt->vctxt.error) ctxt->vctxt.error = filt->vctxt.error;
    if (filt->vctxt.warning) ctxt->vctxt.warning = filt->vctxt.warning;
    if (filt->vctxt.userData) ctxt->vctxt.userData = filt->vctxt.userData;
    if (filt == NULL) return;
    ctxt->sax = filt->oldsax;
    ctxt->userData = filt->ctx;
    if (filt->sax) edir_free(filt->sax);
    edir_free(filt);
}

/**
 * Replacement entity loader that loads statically embedded DTD's.
 * <P>
 * The 'dtdembed' program is used at compile time to embed one or
 * more DTDs as static character arrays.  This function looks up
 * the DTDs by the basename of their original filename, and if found,
 * returns a libxml input stream object.
 * 
 * @param URL  [in] The XML URL for the DTD to load.
 * @param ID   [in] The XML ID for the DTD to load. May be NULL.
 * @param ctxt [in] A libxml parsing context pointer.
 * @return A libxml input stream if successful, or NULL if unsuccessful.
 */
static xmlParserInputPtr
edir_conf_xml_embedded_entity_loader(const char *URL, const char *ID,
                                     xmlParserCtxtPtr ctxt)
{
    static char		*funcName = "edir_conf_xml_embedded_entity_loader";
    xmlParserInputPtr	ret = NULL;
    int			i;
    char		*baseURL;
    EDIR_DEBUG_FUNCS_IN(("func: %s(\"%s\", \"%s\", "EDIR_PTR")", funcName, \
			URL ? URL : "(null)", ID ? ID : "(null)", ctxt));

    edir_dtd_init();

    if (URL == NULL) {
	if ((ctxt->sax != NULL) && (ctxt->sax->warning != NULL)) {
            ctxt->sax->warning(ctxt, "failed to load embedded entity \"%s\"\n",
                               ID ? ID : "(null)" );
	}
	EDIR_DEBUG_FUNCS_OUT(("func: %s() returns NULL", funcName));
        return NULL;
    }

    baseURL = (char *)URL;
    if (strrchr(baseURL, '/')) baseURL = strrchr(baseURL, '/') + 1;
    if (strrchr(baseURL, '\\')) baseURL = strrchr(baseURL, '\\') + 1;

    for (i=0; i < edir_dtd_count; i++) {
	EDIR_DEBUG_EVERYTHING(("Checking %s against %s", \
			    baseURL ? baseURL : "(null)", \
			    edir_dtd_names[i] ? edir_dtd_names[i] : "(null)"));
	if (!edir_strcasecmp(baseURL, edir_dtd_names[i])) {
	    EDIR_DEBUG_EVERYTHING(("Found DTD!"));
	    ret = xmlNewStringInputStream(ctxt,
                                     (unsigned char *)edir_dtd_bufs[i]);
	    if (ret != NULL) return ret;
	}
    }
    if ((ctxt->sax != NULL) && (ctxt->sax->warning != NULL)) {
	ctxt->sax->warning(ctxt, "failed to load embedded entity \"%s\"\n",
			   URL ? URL : "(null)" );
    }
    EDIR_DEBUG_FUNCS_OUT(("func: %s() returns NULL", funcName));
    return NULL;
}

/**
 * The function that handles all libxml parsing and validation error messages.
 * <P>
 * This function is called if errors are encountered and the Enhydra Director
 * error filter has been enabled for the current parsing context.
 * 
 * @param ctx [in,out] Opaque pointer to a filter structure.
 * @param msg [in]     Printf-style format string.
 * @param ... [in]     Printf-style variadic argument list.
 */
static void
edir_conf_xml_error_func(void *ctx, const char *msg, ...)
{
    edir_conf_xml_sax_filter_t	*filt = (edir_conf_xml_sax_filter_t *) ctx;
    edir_conf_xml_errdata_t	*errData;
    char			buf[512], *cp, *ep;
    va_list			ap;
    errData = (edir_conf_xml_errdata_t *) filt->data;
    if (errData == NULL) return;
    va_start(ap, msg);
    edir_vsnprintf(buf, 511, msg, ap);
    va_end(ap);
    buf[511] = '\0';
    cp = buf;
    if ((ep = strrchr(cp, '\n')) && (*(ep+1) == '\0')) {
	if ((ep > cp) && (*(ep-1) == '\r')) ep--;
	*ep = '\0';
    }
    if (errData->sbuf == NULL) {
	errData->sbuf = edir_sbuf_create();
	edir_sbuf_appendf(errData->sbuf, "ERROR: Config file \"%s\":",
			  filt->ctxt->input->filename);
    }
    edir_sbuf_appendf(errData->sbuf, "\nLine %d: %s",
		      filt->ctxt->input->line, buf);
}

/**
 * Place an Enhydra Director error filter on a libxml parsing context.
 * <P>
 * This error filter captures any parsing or validation error messages
 * and writes the messages to a string buffer object.  This object
 * can then be used to produce an <code>edir_exception_t</code> object
 * to throw.
 * 
 * @param ctxt [in,out] The libxml parsing context to filter.
 * @return A pointer to a structure containing the sbuf.  This sbuf is NULL
 * 	until the first error message is set.
 */
extern edir_conf_xml_errdata_t *
edir_xml_set_error_filter(xmlParserCtxt *ctxt)
{
    xmlSAXHandler		*filter = NULL;
    edir_conf_xml_errdata_t	*errorData = NULL;
    filter = (xmlSAXHandlerPtr) edir_malloc(sizeof(xmlSAXHandler));
    memset((char *)filter, 0, sizeof(xmlSAXHandler));
    filter->error = edir_conf_xml_error_func;
    filter->warning = edir_conf_xml_error_func;
    filter->fatalError = edir_conf_xml_error_func;
    errorData = (edir_conf_xml_errdata_t *)
                edir_malloc(sizeof(edir_conf_xml_errdata_t));
    memset((char *)errorData, 0, sizeof(edir_conf_xml_errdata_t));
    edir_xml_sax_install_filter(ctxt, filter, (void *) errorData);
    return errorData;
}

/**
 * Clear the Enhydra Director error filter form this libxml
 * parsing context.
 * <P>
 * Errors will again go to standard error.
 * 
 * @param ctxt [in,out] The parsing context to restore.
 */
extern void
edir_xml_clear_error_filter(xmlParserCtxt *ctxt)
{
    edir_conf_xml_sax_filter_t		*filt;
    edir_conf_xml_errdata_t		*errorData = NULL;
    filt = (edir_conf_xml_sax_filter_t *)(ctxt->userData);
    errorData = (edir_conf_xml_errdata_t *) filt->data;
    if (errorData) {
	if (errorData->sbuf) edir_sbuf_free(errorData->sbuf);
	edir_free(errorData);
    }
    edir_xml_sax_remove_filter(ctxt);
}

/**
 * Initialize the Enhydra Director XML layer.
 * <P>
 * Sets up the external entity loader to load entities statically
 * embedded in the current program at compile time.
 * <P>
 * Also enables full DTD validation when parsing.
 */
extern void
edir_xml_init()
{
    xmlSetExternalEntityLoader(edir_conf_xml_embedded_entity_loader);
    xmlDoValidityCheckingDefaultValue = 1;
}

extern void *
edir_xml_malloc(int sz)
{
    return edir_malloc((int32) sz);
}

extern void
edir_xml_free(void *p)
{
    edir_free(p);
}

extern void *
edir_xml_realloc(void *p, int sz)
{
    return edir_realloc(p, (int32) sz);
}

extern char *
edir_xml_strdup(const char *s)
{
    return edir_strdup(s);
}
