/*
 * Copyright (c) 2001 Silver Egg Technology
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without 
 * restriction, including without limitation the rights to use, 
 * copy, modify, merge, publish, distribute, sublicense, and/or 
 * sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following 
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 *
 * $Log: wrapper.h,v $
 * Revision 1.1  2003/04/14 12:32:14  sinisa
 * *** empty log message ***
 *
 * Revision 1.1  2003/02/28 14:51:02  sinisa
 * *** empty log message ***
 *
 * Revision 1.21  2002/12/06 18:48:39  mortenson
 * Add a property, wrapper.ntservice.interactive, which makes it possible to
 * control whether or not the Java process can gain access to the desktop while
 * it is running as an NT service.
 *
 * Revision 1.20  2002/12/04 06:28:38  mortenson
 * Add the ability to specify an account name and password when installing an
 * NT service.
 *
 * Revision 1.19  2002/10/10 03:20:09  mortenson
 * Fix a problem where the Wrapper would not respond to exit requests while
 * pausing between JVM invocations.
 *
 * Revision 1.18  2002/09/17 13:16:22  mortenson
 * Added a property to control the delay between JVM invocations.
 *
 * Revision 1.17  2002/09/10 16:17:26  mortenson
 * Get rid of tabs in files.  No other changes.
 *
 * Revision 1.16  2002/09/09 17:19:46  mortenson
 * Add ability to log to specific log levels from within the Wrapper.
 *
 * Revision 1.15  2002/08/11 05:32:44  mortenson
 * Make it possible for the user to configure the restart count and time via
 * the wrapper.max_failed_invocations and wrapper.successful_invocation_time
 * properties.  It was always 5 restarts within 30 seconds.
 * Added the ability to configure the JVM exit timeout via the
 * wrapper.jvm_exit.timeout property.  It was always 5.
 *
 * Revision 1.14  2002/07/19 02:06:12  mortenson
 * Added a new property: wrapper.cpu.timeout to control the cpu timeout added in
 * v2.2.7
 *
 * Revision 1.13  2002/05/23 12:42:41  rybesh
 * fixed logger initialization on unix
 *
 * Revision 1.12  2002/05/17 09:10:16  mortenson
 * Add a wrapper.shutdown.timeout property.
 *
 * Revision 1.11  2002/05/16 04:51:18  mortenson
 * Add a debug message stating which thread lead to System.exit being called
 *   via a call to shutdown.
 *
 * Revision 1.10  2002/05/07 05:46:36  mortenson
 * Add the ability to set the priority at which the wrapper is run under NT systems.
 *
 * Revision 1.9  2002/03/07 10:05:48  rybesh
 * fixed some miscellaneous unix compile errors
 *
 * Revision 1.8  2002/03/07 09:23:25  mortenson
 * Go through and change the style of comments that we use so that they will not
 * cause compiler errors on older unix compilers.
 *
 * Revision 1.7  2002/03/07 08:10:13  mortenson
 * Add support for Thread Dumping
 * Fix a problem locating java on the path.
 *
 * Revision 1.6  2002/01/27 19:33:55  spocke
 * Added support for the service description property.
 *
 * Revision 1.5  2002/01/24 09:43:56  mortenson
 * Added new Logger code which allows log levels.
 *
 * Revision 1.4  2002/01/24 05:48:48  mortenson
 * Remove old commented code.
 *
 * Revision 1.3  2001/12/11 05:19:39  mortenson
 * Added the ablility to format and/or disable file logging and output to
 * the console.
 *
 * Revision 1.2  2001/12/06 09:36:24  mortenson
 * Docs changes, Added sample apps, Fixed some problems with
 * relative paths  (See revisions.txt)
 *
 * Revision 1.1.1.1  2001/11/07 08:54:20  mortenson
 * no message
 *
 */

#ifndef _WRAPPER_H
#define _WRAPPER_H

#ifdef WIN32
#include <winsock.h>
#else
#include <time.h>
#define u_short unsigned short
#endif

#include "property.h"

#define WRAPPER_WSTATE_STARTING  51 /* Wrapper is starting.  Remains in this state
                                     *  until the JVM enters the STARTED state or
                                     *  the wrapper jumps into the STOPPING state
                                     *  in response to the JVM application asking
                                     *  to shut down. */
#define WRAPPER_WSTATE_STARTED   52 /* The JVM has entered the STARTED state.
                                     *  The wrapper will remain in this state
                                     *  until the wrapper decides to shut down.
                                     *  This is true even when the JVM process
                                     *  is being restarted. */
#define WRAPPER_WSTATE_STOPPING  53 /* The wrapper is shutting down.  Will be in
                                     *  this state until the JVM enters the DOWN
                                     *  state. */
#define WRAPPER_WSTATE_STOPPED   54 /* The wrapper enters this state just before
                                     *  it exits. */


#define WRAPPER_JSTATE_DOWN      71 /* JVM is confirmed to be down.  This is the 
                                     *  initial state and the state after the JVM
                                     *  process has gone away. */
#define WRAPPER_JSTATE_LAUNCH    72 /* Set from the DOWN state to launch a JVM.  The
                                     *  timeout will be the time to actually launch
                                     *  the JVM after any required delay. */
#define WRAPPER_JSTATE_LAUNCHING 73 /* JVM was launched, but has not yet responded.
                                     *  Must enter the LAUNCHED state before <t>
                                     *  or the JVM will be killed. */
#define WRAPPER_JSTATE_LAUNCHED  74 /* JVM was launched, and responed to a ping. */
#define WRAPPER_JSTATE_STARTING  75 /* JVM has been asked to start.  Must enter the
                                     *  STARTED state before <t> or the JVM will be
                                     *  killed. */
#define WRAPPER_JSTATE_STARTED   76 /* JVM has responded that it is running.  Must
                                     *  respond to a ping by <t> or the JVM will
                                     *  be killed. */
#define WRAPPER_JSTATE_STOPPING  77 /* JVM was sent a stop command, but has not yet
                                     *  responded.  Must enter the STOPPED state
                                     *  and exit before <t> or the JVM will be killed. */
#define WRAPPER_JSTATE_STOPPED   78 /* JVM has responed that it is stopped. */


/* Type definitions */
typedef struct WrapperConfig WrapperConfig;
struct WrapperConfig {
    u_short port;                   /* Port number which the Wrapper is configured to be listening on */
    u_short actualPort;             /* Port number which the Wrapper is actually listening on */
    int     sock;                   /* Socket number. if open. */
    char    *configFile;            /* Name of the config file */
#ifdef WIN32
    char    *jvmCommand;            /* Command used to launch the JVM */
#else /* UNIX */
    char    **jvmCommand;           /* Command used to launch the JVM */
#endif
    char    key[17];                /* Key which the JVM uses to authorize connections. (16 digits + \0) */
    int     isConsole;              /* TRUE if the wrapper was launched as a console. */
    int     cpuTimeout;             /* Number of seconds without CPU before the JVM will issue a warning and extend timeouts */
    int     startupTimeout;         /* Number of seconds the wrapper will wait for a JVM to startup */
    int     pingTimeout;            /* Number of seconds the wrapper will wait for a JVM to reply to a ping */
    int     shutdownTimeout;        /* Number of seconds the wrapper will wait for a JVM to shutdown */
    int     jvmExitTimeout;         /* Number of seconds the wrapper will wait for a JVM to process to terminate */
    int     wState;                 /* The current state of the wrapper */
    int     jState;                 /* The current state of the jvm */
    time_t  jStateTimeout;          /* Time until which the current jState is valid */
    time_t  lastPingTime;           /* Time that the last ping was sent */
    int     isDebugging;            /* TRUE if set in the config file */
    int     isStateOutputEnabled;   /* TRUE if set in the config file.  Shows output on the state of the state engine. */
    int     isShutdownHookDisabled; /* TRUE if set in the config file */
    int     exitCode;               /* Code which the wrapper will exit with */
    int     exitRequested;          /* Non-zero if another thread has requested that the wrapper and JVM be shutdown */
    int     exitAcknowledged;       /* Non-zero if the main thread has acknowledged the exit request */
    int     restartRequested;       /* Non-zero if another thread has requested that the JVM be restarted */
    int     jvmRestarts;            /* Number of times that a JVM has been launched since the wrapper was started. */
    int     restartDelay;           /* Delay in seconds before restarting a new JVM. */
    int     requestThreadDumpOnFailedJVMExit; /* TRUE if the JVM should be asked to dump its state when it fails to halt on request. */
    time_t  jvmLaunchTime;          /* The time that the previous or current JVM was launched. */
    int     failedInvocationCount;  /* The number of times that the JVM exited in less than successfulInvocationTime in a row. */
    int     successfulInvocationTime;/* Amount of time that a new JVM must be running so that the invocation will be considered to have been a success, leading to a reset of the restart count. */
    int     maxFailedInvocations;   /* Maximum number of failed invocations in a row before the Wrapper will give up and exit. */

#ifdef WIN32
    char    *ntServiceName;         /* Name of the NT Service */
    char    *ntServiceDisplayName;  /* Display name of the NT Service */
    char    *ntServiceDescription;  /* Description for service in Win2k and XP */
    char    *ntServiceDependencies; /* List of Dependencies */
    int     ntServiceStartType;     /* Mode in which the Service is installed. 
                                     * {SERVICE_AUTO_START | SERVICE_DEMAND_START} */
    DWORD   ntServicePriorityClass; /* Priority at which the Wrapper and its JVMS will run.
                                     * {HIGH_PRIORITY_CLASS | IDLE_PRIORITY_CLASS | NORMAL_PRIORITY_CLASS | REALTIME_PRIORITY_CLASS} */
	char    *ntServiceAccount;      /* Account name to use when running as a service.  NULL to use the LocalSystem account. */
	char    *ntServicePassword;     /* Password to use when running as a service.  NULL means no password. */
	int     ntServiceInteractive;   /* Should the service be allowed to interact with the desktop? */
#endif

#ifdef SOLARIS
    char    *pidFilename;           /* Name of file to store wrapper pid in */
#endif
};

#define WRAPPER_MSG_START         (char)100
#define WRAPPER_MSG_STOP          (char)101
#define WRAPPER_MSG_RESTART       (char)102
#define WRAPPER_MSG_PING          (char)103
#define WRAPPER_MSG_STOP_PENDING  (char)104
#define WRAPPER_MSG_START_PENDING (char)105
#define WRAPPER_MSG_STARTED       (char)106
#define WRAPPER_MSG_STOPPED       (char)107
#define WRAPPER_MSG_KEY           (char)110
#define WRAPPER_MSG_BADKEY        (char)111
#define WRAPPER_MSG_LOW_LOG_LEVEL (char)112

/** Log commands are actually 116 + the LOG LEVEL. */
#define WRAPPER_MSG_LOG           (char)116

#define WRAPPER_PROCESS_DOWN      200
#define WRAPPER_PROCESS_UP        201

extern WrapperConfig *wrapperData;
extern Properties    *properties;

extern char wrapperClasspathSeparator;

/* Protocol Functions */
extern void wrapperProtocolStartServer();
extern void wrapperProtocolStopServer();
extern void wrapperProtocolOpen();
extern void wrapperProtocolClose();
extern int wrapperProtocolFunction(char function, const char *message);
extern int wrapperProtocolRead();

/******************************************************************************
 * Utility Functions
 *****************************************************************************/
#ifdef WIN32
extern char** wrapperGetSystemPath();
#endif

extern int wrapperCheckRestartTimeOK();

/**
 * command is a pointer to a pointer of an array of character strings.
 * length is the number of strings in the above array.
 */
extern void wrapperBuildJavaCommandArray(char ***strings, int *length, int addQuotes);
extern void wrapperFreeJavaCommandArray(char **strings, int length);

extern void wrapperInitializeLogging();

/******************************************************************************
 * Platform specific methods
 *****************************************************************************/

/**
 * Gets the error code for the last operation that failed.
 */
extern int wrapperGetLastError();

/**
 * Execute initialization code to get the wrapper set up.
 */
extern int wrapperInitialize();

/**
 * Execute clean up code in preparation for shutdown
 */
extern void wrapperCleanup();

/**
 * Reports the status of the wrapper to the service manager
 * Possible status values:
 *   WRAPPER_WSTATE_STARTING
 *   WRAPPER_WSTATE_STARTED
 *   WRAPPER_WSTATE_STOPPING
 *   WRAPPER_WSTATE_STOPPED
 */
extern void wrapperReportStatus(int status, int errorCode, int waitHint);

/**
 * Read and process any output from the child JVM Process.
 * Most output should be logged to the wrapper log file.
 */
extern void wrapperReadChildOutput();

/**
 * Checks on the status of the JVM Process.
 * Returns WRAPPER_PROCESS_UP or WRAPPER_PROCESS_DOWN
 */
extern int wrapperGetProcessStatus();

/**
 * Kill the JVM Process immediately and set the JVM State to
 *  WRAPPER_JSTATE_DOWN
 */
extern void wrapperKillProcess();

/**
 * Pauses before launching a new JVM if necessary.
 */
extern void wrapperPauseBeforeExecute();

/**
 * Launches a JVM process and store it internally
 */
extern void wrapperExecute();

/******************************************************************************
 * Wrapper inner methods.
 *****************************************************************************/
/**
 * Launch the wrapper as a console application.
 */
extern int wrapperRunConsole();

/**
 * Launch the wrapper as a service application.
 */
extern int wrapperRunService();

/**
 * Used to ask the state engine to shut down the JVM and Wrapper
 */
extern void wrapperStopProcess(int exitCode);

/**
 * Used to ask the state engine to shut down the JVM.
 */
extern void wrapperRestartProcess();

/**
 * The main event loop for the wrapper.  Handles all state changes and events.
 */
extern void wrapperEventLoop();

extern void wrapperBuildKey();
extern void wrapperBuildJavaCommand();
extern int  wrapperLoadConfiguration();

/******************************************************************************
 * Protocol callback functions
 *****************************************************************************/
extern void wrapperLogSignalled(int logLevel, char *msg);
extern void wrapperKeyRegistered(char *key);
extern void wrapperPingResponded();
extern void wrapperStopRequested(int exitCode);
extern void wrapperRestartRequested();
extern void wrapperStopPendingSignalled(int waitHint);
extern void wrapperStoppedSignalled();
extern void wrapperStartPendingSignalled(int waitHint);
extern void wrapperStartedSignalled();

#endif
