
/**
 * Enhydra Director configuration info.  
 */
typedef struct edir_nsapi_cfg {
    /*** Bookeeping to keep track of where in Apache config we are ***/
    int   cfg_Mode;	   /* Per-server or Per-directory config?       */
    int   inited;	   /* Whether we've initialized yet.            */
    int   cfg_Inheritable; /* Can virt. servers inherit this from main? */
//    char  *cfg_Server;	   /* Server name if per-virtual-host config    */
    char  *cfg_Dir;	   /* Dir path if per-directory config          */
    /*** Per-instance config for each Enhydra Director setup. ***/
    int32 inBufSize;
    int32 outBufSize;
    char  *configFile;     /* Configuration file for this instance      */
    char  *dataFile;       /* Optional place to store scoreboard data   */
                           /*   NOTE: This _may_ be an IPC key #        */
    char  *lockFile;       /* Optional place to locate locks/mutexes    */
                           /*   NOTE: This _may_ be an IPC key #        */
    char  *pathToDaemon;   /* Optional full path to daemon executable.  */
    char  *sessionIdPrefix;/* Prefix of Session ID cookie or component  */
//    edir_load_handles_t
//          *handles;        /* Scoreboard handles. */
} edir_nsapi_cfg_t;

/**
 * Structure to pass to edir_parse_uri() to receive important URI
 * fragments.
 */
typedef struct edir_uri_data {
    char           *uriBuf;	   /* destroyed uri buffer */
    char           *pathInfo;	   /* Stripped, normalized URI path. */ 
    char           *pathParams;	   /* Full path parameters string, if any */
    char           *sessionID;	   /* Session ID, if any.  */
    char           *queryString;   /* Query String, if any. */
    edir_kvtable_t *cookies;	   /* Hack: let cookie getter set sessionID */
    char           *virtHost;      /* Hack: The virtual host this uri connects to */
} edir_uri_data_t;

/**
 * Structure used to pass important info into various header handling routines
 */
typedef struct edir_nsapi_header_handler {
    edir_connection_t *conn;
    octet	      *outBuf;
    int		       outBufSize;
    octet	      *inBuf;
    int		       inBufSize;
} edir_nsapi_header_handler_t;

#define DIR_CONFIG(R) \
	((edir_nsapi_cfg_t *) \
	  (ap_get_module_config((R)->per_dir_config, &enhydra_director_module)));

#define SERVER_CONFIG(S) \
	((edir_nsapi_cfg_t *) \
	  (ap_get_module_config((S)->module_config, &enhydra_director_module)));

#define REQ_CONFIG(R) \
	((edir_nsapi_cfg_t *) \
	  (ap_get_module_config((R)->request_config, &enhydra_director_module)));

extern void edir_nsapi_log_init();
extern void edir_nsapi_log_set_session(Session *sn, Request *rq);

#define EDIR_NSAPI_PREBUF_SIZE		(2048)
#define EDIR_NSAPI_INBUF_SIZE		(4096)
#define EDIR_NSAPI_OUTBUF_SIZE		(4096)
 
#define EDIR_NSAPI_CFG_MODE_SERVER	(1)
#define EDIR_NSAPI_CFG_MODE_DIR		(2)

extern edir_nsapi_cfg_t       *globalCfg;
extern int32                  *refCounts;
extern int32                   numServers;
extern edir_load_sb_offsets_t *scoreboard;

