/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 * $Id: dtdembed.c,v 1.1 2003/04/14 12:26:04 sinisa Exp $
 */

#include "edir.h"

#ifdef WIN32
extern void exit();
#endif

#define DTD_BUFSIZE	1000

static char buf[DTD_BUFSIZE];

static char *outFile = NULL;

static char *inFile = NULL;

static FILE *in = NULL;

static FILE *out = NULL;

static char *header = ""\
"/*\n"\
" * Enhydra Java Application Server \n"\
" * The Initial Developer of the Original Code is Lutris Technologies Inc. \n"\
" * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies\n"\
" * Inc. \n"\
" * All Rights Reserved. \n"\
" *\n"\
" * The contents of this file are subject to the Enhydra Public License Version\n"\
" * 1.0 (the \"License\"); you may not use this file except in compliance with the\n"\
" * License. You may obtain a copy of the License at\n"\
" * http://www.enhydra.org/software/license/epl.html \n"\
" *\n"\
" * Software distributed under the License is distributed on an \"AS IS\" basis,\n"\
" * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the\n"\
" * License for the specific language governing rights and limitations under the\n"\
" * License. \n"\
" *\n"\
" * MACHINE GENERATED FILE: DO NOT EDIT!\n"\
" */\n\n";

static char *
pathbasename(char *path)
{
    char *base;
    if ((!path) || (!(*path))) path="gendtd";
    for (base=path; *base; ) base++;
    for (--base; (base > path) && (*base != '/') && (*base != '\\'); base--);
    if ((*base == '/') || (*base == '\\')) base++;
    return base;
}

static void
usage(char *progname)
{
    fprintf(stderr, "Usage: %s <outFile> <inFile>", pathbasename(progname));
    exit(1);
}

void
cleanup(int status)
{
    if (out) fclose(out);
    if (outFile) unlink(outFile);
    exit(status);
}

main(int argc, char *argv[])
{
    int		i, j, linepos, len;

    if (argc < 3) usage(argv[0]);

    outFile = argv[1];
    if ((out = fopen(outFile, "w")) == NULL) {
	perror(outFile);
	cleanup(1);
    }

    fprintf(out, "%s", header);
    fflush(out);

    for (i=2; i<argc; i++) {
	inFile = argv[i];
	fprintf(out, "static char *name%d = \"%s\";\n", i - 2, pathbasename(inFile));
	fprintf(out, "static char dtd%d[] = {\n", i - 2);

	linepos = 0;

	if ((in = fopen(inFile, "r")) == NULL) {
	    perror(inFile);
	    cleanup(1);
	}
	while ((len = fread(buf, 1, DTD_BUFSIZE, in)) > 0) {
	    for (j=0; j<len; j++) {
		fprintf(out, "0x%02x,", ((int)buf[j]) & 0xff);
		linepos++;
		if (linepos > 15) {
		    fprintf(out, "\n");
		    linepos = 0;
		}
	    }
	}
	fprintf(out, "0x00\n};\n\n");
	fclose(in);
	fflush(out);
    }

    fprintf(out, "char *edir_dtd_names[%d];\n\n", argc - 1);

    fprintf(out, "char *edir_dtd_bufs[%d];\n\n", argc - 1);

    fprintf(out, "int edir_dtd_count = %d;\n\n", argc - 2);

    fprintf(out, "extern void\nedir_dtd_init()\n{\n");
    for (i = 2; i < argc; i++) {
	fprintf(out, "    edir_dtd_names[%d] = name%d;\n", i-2, i-2);
	fprintf(out, "    edir_dtd_bufs[%d] = dtd%d;\n", i-2, i-2);
    }	
    fprintf(out, "    edir_dtd_names[%d] = (char *)0;\n", argc-2);
    fprintf(out, "    edir_dtd_bufs[%d] = (char *)0;\n", argc-2);
    fprintf(out, "}\n\n");

    fflush(out);
    fclose(out);

    exit(0);
}
